@@ -27,12 +27,13 @@ import six
 
 import oslo_messaging
 from oslo_messaging._i18n import _
-from oslo_messaging._i18n import _LE
+from oslo_messaging._i18n import _LE, _LW
 from oslo_messaging import _utils as utils
 
 LOG = logging.getLogger(__name__)
 
 _EXCEPTIONS_MODULE = 'exceptions' if six.PY2 else 'builtins'
+_EXCEPTIONS_MODULES = ['exceptions', 'builtins']
 
 
 '''RPC Envelope Version.
@@ -88,7 +89,7 @@ class RPCException(Exception):
                 # log the issue and the kwargs
                 LOG.exception(_LE('Exception in string format operation, '
                                   'kwargs are:'))
-                for name, value in six.iteritems(kwargs):
+                for name, value in kwargs.items():
                     LOG.error("%s: %s", name, value)
                 # at least get the core message out if something happened
                 message = self.msg_fmt
@@ -208,6 +209,13 @@ def deserialize_remote_exception(data, allowed_remote_exmods):
     name = failure.get('class')
     module = failure.get('module')
 
+    # the remote service which raised the given exception might have a
+    # different python version than the caller. For example, the caller might
+    # run python 2.7, while the remote service might run python 3.4. Thus,
+    # the exception module will be "builtins" instead of "exceptions".
+    if module in _EXCEPTIONS_MODULES:
+        module = _EXCEPTIONS_MODULE
+
     # NOTE(ameade): We DO NOT want to allow just any module to be imported, in
     # order to prevent arbitrary code execution.
     if module != _EXCEPTIONS_MODULE and module not in allowed_remote_exmods:
@@ -221,7 +229,9 @@ def deserialize_remote_exception(data, allowed_remote_exmods):
             raise TypeError("Can only deserialize Exceptions")
 
         failure = klass(*failure.get('args', []), **failure.get('kwargs', {}))
-    except (AttributeError, TypeError, ImportError):
+    except (AttributeError, TypeError, ImportError) as error:
+        LOG.warning(_LW("Failed to rebuild remote exception due to error: %s"),
+                    six.text_type(error))
         return oslo_messaging.RemoteError(name, failure.get('message'), trace)
 
     ex_type = type(failure)
@@ -339,6 +349,9 @@ class DecayingTimer(object):
     def start(self):
         self._watch.start()
 
+    def restart(self):
+        self._watch.restart()
+
     def check_return(self, timeout_callback=None, *args, **kwargs):
         maximum = kwargs.pop('maximum', None)
         left = self._watch.leftover(return_none=True)
@@ -360,7 +373,7 @@ class DecayingTimer(object):
 # greenthread.
 # So, a connection cannot be shared between thread/greenthread and
 # this two variables permit to define the purpose of the connection
-# to allow drivers to add special handling if needed (like heatbeat).
+# to allow drivers to add special handling if needed (like heartbeat).
 # amqp drivers create 3 kind of connections:
 # * driver.listen*(): each call create a new 'PURPOSE_LISTEN' connection
 # * driver.send*(): a pool of 'PURPOSE_SEND' connections is used
@@ -413,16 +426,18 @@ class ConnectionContext(Connection):
                     LOG.exception(_LE("Fail to reset the connection, drop it"))
                     try:
                         self.connection.close()
-                    except Exception:
-                        pass
+                    except Exception as exc:
+                        LOG.debug("pooled conn close failure (ignored): %s",
+                                  str(exc))
                     self.connection = self.connection_pool.create()
                 finally:
                     self.connection_pool.put(self.connection)
             else:
                 try:
                     self.connection.close()
-                except Exception:
-                    pass
+                except Exception as exc:
+                    LOG.debug("pooled conn close failure (ignored): %s",
+                              str(exc))
             self.connection = None
 
     def __exit__(self, exc_type, exc_value, tb):
@@ -452,13 +467,21 @@ class ConfigOptsProxy(collections.Mapping):
     and valid) override corresponding values from the configuration.
     """
 
-    def __init__(self, conf, url):
+    def __init__(self, conf, url, group):
         self._conf = conf
         self._url = url
+        self._group = group
+        self._validate_query()
+
+    def _validate_query(self):
+        for name in self._url.query:
+            self.GroupAttrProxy(self._conf, self._group,
+                                self._conf[self._group],
+                                self._url)[name]
 
     def __getattr__(self, name):
         value = getattr(self._conf, name)
-        if isinstance(value, self._conf.GroupAttr):
+        if isinstance(value, self._conf.GroupAttr) and name == self._group:
             return self.GroupAttrProxy(self._conf, name, value, self._url)
         return value
 
