@@ -45,6 +45,11 @@ controller = importutils.try_import(
 )
 LOG = logging.getLogger(__name__)
 
+# Build/Decode RPC Response messages
+# Body Format - json string containing a map with keys:
+# 'failure' - (optional) serialized exception from remote
+# 'response' - (if no failure provided) data returned by call
+
 
 def marshal_response(reply, failure):
     # TODO(grs): do replies have a context?
@@ -70,7 +75,14 @@ def unmarshal_response(message, allowed):
     return data.get("response")
 
 
-def marshal_request(request, context, envelope):
+# Build/Decode RPC Request and Notification messages
+# Body Format: json string containing a map with keys:
+# 'request' - possibly serialized application data
+# 'context' - context provided by the application
+# 'call_monitor_timeout' - optional time in seconds for RPC call monitoring
+
+def marshal_request(request, context, envelope=False,
+                    call_monitor_timeout=None):
     # NOTE(flaper87): Set inferred to True since rabbitmq-amqp-1.0 doesn't
     # have support for vbin8.
     msg = proton.Message(inferred=True)
@@ -80,6 +92,8 @@ def marshal_request(request, context, envelope):
         "request": request,
         "context": context
     }
+    if call_monitor_timeout is not None:
+        data["call_monitor_timeout"] = call_monitor_timeout
     msg.body = jsonutils.dumps(data)
     return msg
 
@@ -87,36 +101,55 @@ def marshal_request(request, context, envelope):
 def unmarshal_request(message):
     data = jsonutils.loads(message.body)
     msg = common.deserialize_msg(data.get("request"))
-    return (msg, data.get("context"))
+    return (msg, data.get("context"), data.get("call_monitor_timeout"))
 
 
 class ProtonIncomingMessage(base.RpcIncomingMessage):
-    def __init__(self, listener, ctxt, request, message, disposition):
+    def __init__(self, listener, message, disposition):
+        request, ctxt, client_timeout = unmarshal_request(message)
         super(ProtonIncomingMessage, self).__init__(ctxt, request)
         self.listener = listener
+        self.client_timeout = client_timeout
         self._reply_to = message.reply_to
         self._correlation_id = message.id
         self._disposition = disposition
 
+    def heartbeat(self):
+        # heartbeats are sent "worst effort": non-blocking, no retries,
+        # pre-settled (no blocking for acks). We don't want the server thread
+        # being blocked because it is unable to send a heartbeat.
+        if not self._reply_to:
+            LOG.warning("Cannot send RPC heartbeat: no reply-to provided")
+            return
+        # send a null msg (no body). This will cause the client to simply reset
+        # its timeout (the null message is dropped).  Use time-to-live to
+        # prevent stale heartbeats from building up on the message bus
+        msg = proton.Message()
+        msg.correlation_id = self._correlation_id
+        msg.ttl = self.client_timeout
+        task = controller.SendTask("RPC KeepAlive", msg, self._reply_to,
+                                   deadline=None, retry=0, wait_for_ack=False)
+        self.listener.driver._ctrl.add_task(task)
+        task.wait()
+
     def reply(self, reply=None, failure=None):
         """Schedule an RPCReplyTask to send the reply."""
         if self._reply_to:
             response = marshal_response(reply, failure)
             response.correlation_id = self._correlation_id
-            LOG.debug("Sending RPC reply to %s (%s)", self._reply_to,
-                      self._correlation_id)
             driver = self.listener.driver
             deadline = compute_timeout(driver._default_reply_timeout)
+            ack = not driver._pre_settle_reply
             task = controller.SendTask("RPC Reply", response, self._reply_to,
                                        # analogous to kombu missing dest t/o:
                                        deadline,
-                                       retry=0,
-                                       wait_for_ack=True)
+                                       retry=driver._default_reply_retry,
+                                       wait_for_ack=ack)
             driver._ctrl.add_task(task)
             rc = task.wait()
             if rc:
                 # something failed.  Not much we can do at this point but log
-                LOG.debug("Reply failed to send: %s", str(rc))
+                LOG.debug("RPC Reply failed to send: %s", str(rc))
         else:
             LOG.debug("Ignoring reply as no reply address available")
 
@@ -125,18 +158,12 @@ class ProtonIncomingMessage(base.RpcIncomingMessage):
         task = controller.MessageDispositionTask(self._disposition,
                                                  released=False)
         self.listener.driver._ctrl.add_task(task)
-        rc = task.wait()
-        if rc:
-            LOG.debug("Message acknowledge failed: %s", str(rc))
 
     def requeue(self):
         """Schedule a MessageDispositionTask to release the message"""
         task = controller.MessageDispositionTask(self._disposition,
                                                  released=True)
         self.listener.driver._ctrl.add_task(task)
-        rc = task.wait()
-        if rc:
-            LOG.debug("Message requeue failed: %s", str(rc))
 
 
 class Queue(object):
@@ -183,11 +210,9 @@ class ProtonListener(base.PollStyleListener):
         qentry = self.incoming.pop(timeout)
         if qentry is None:
             return None
-        message = qentry['message']
-        request, ctxt = unmarshal_request(message)
-        disposition = qentry['disposition']
-        LOG.debug("poll: message received")
-        return ProtonIncomingMessage(self, ctxt, request, message, disposition)
+        return ProtonIncomingMessage(self,
+                                     qentry['message'],
+                                     qentry['disposition'])
 
 
 class ProtonDriver(base.BaseDriver):
@@ -208,8 +233,8 @@ class ProtonDriver(base.BaseDriver):
                                  title='AMQP 1.0 driver options')
         conf.register_group(opt_group)
         conf.register_opts(opts.amqp1_opts, group=opt_group)
+        conf = common.ConfigOptsProxy(conf, url, opt_group.name)
 
-        self._hosts = url.hosts
         self._conf = conf
         self._default_exchange = default_exchange
 
@@ -224,6 +249,19 @@ class ProtonDriver(base.BaseDriver):
         self._default_reply_timeout = opt_name.default_reply_timeout
         self._default_send_timeout = opt_name.default_send_timeout
         self._default_notify_timeout = opt_name.default_notify_timeout
+        self._default_reply_retry = opt_name.default_reply_retry
+
+        # which message types should be sent pre-settled?
+        ps = [s.lower() for s in opt_name.pre_settled]
+        self._pre_settle_call = 'rpc-call' in ps
+        self._pre_settle_reply = 'rpc-reply' in ps
+        self._pre_settle_cast = 'rpc-cast' in ps
+        self._pre_settle_notify = 'notify' in ps
+        bad_opts = set(ps).difference(['rpc-call', 'rpc-reply',
+                                      'rpc-cast', 'notify'])
+        if bad_opts:
+            LOG.warning(_LW("Ignoring unrecognized pre_settle value(s): %s"),
+                        " ".join(bad_opts))
 
     def _ensure_connect_called(func):
         """Causes a new controller to be created when the messaging service is
@@ -251,7 +289,7 @@ class ProtonDriver(base.BaseDriver):
                         self._ctrl = None
                     # Create a Controller that connects to the messaging
                     # service:
-                    self._ctrl = controller.Controller(self._hosts,
+                    self._ctrl = controller.Controller(self._url,
                                                        self._default_exchange,
                                                        self._conf)
                     self._ctrl.connect()
@@ -260,7 +298,8 @@ class ProtonDriver(base.BaseDriver):
 
     @_ensure_connect_called
     def send(self, target, ctxt, message,
-             wait_for_reply=False, timeout=None, envelope=False,
+             wait_for_reply=False,
+             timeout=None, call_monitor_timeout=None,
              retry=None):
         """Send a message to the given target.
 
@@ -275,32 +314,40 @@ class ProtonDriver(base.BaseDriver):
         :param timeout: raise exception if send does not complete within
                         timeout seconds. None == no timeout.
         :type timeout: float
-        :param envelope: Encapsulate message in an envelope
-        :type envelope: bool
+        :param call_monitor_timeout: Maximum time the client will wait for the
+            call to complete or receive a message heartbeat indicating the
+            remote side is still executing.
+        :type call_monitor_timeout: float
         :param retry: (optional) maximum re-send attempts on recoverable error
                       None or -1 means to retry forever
                       0 means no retry
                       N means N retries
         :type retry: int
-"""
-        request = marshal_request(message, ctxt, envelope)
-        expire = 0
+        """
+        request = marshal_request(message, ctxt, None,
+                                  call_monitor_timeout)
         if timeout:
-            expire = compute_timeout(timeout)  # when the caller times out
-            # amqp uses millisecond time values, timeout is seconds
-            request.ttl = int(timeout * 1000)
-            request.expiry_time = int(expire * 1000)
+            expire = compute_timeout(timeout)
+            request.ttl = timeout
+            request.expiry_time = compute_timeout(timeout)
         else:
             # no timeout provided by application.  If the backend is queueless
             # this could lead to a hang - provide a default to prevent this
             # TODO(kgiusti) only do this if brokerless backend
             expire = compute_timeout(self._default_send_timeout)
-        LOG.debug("Sending message to %s", target)
         if wait_for_reply:
-            task = controller.RPCCallTask(target, request, expire, retry)
+            ack = not self._pre_settle_call
+            if call_monitor_timeout is None:
+                task = controller.RPCCallTask(target, request, expire, retry,
+                                              wait_for_ack=ack)
+            else:
+                task = controller.RPCMonitoredCallTask(target, request, expire,
+                                                       call_monitor_timeout,
+                                                       retry, wait_for_ack=ack)
         else:
+            ack = not self._pre_settle_cast
             task = controller.SendTask("RPC Cast", request, target, expire,
-                                       retry, wait_for_ack=True)
+                                       retry, wait_for_ack=ack)
         self._ctrl.add_task(task)
 
         reply = task.wait()
@@ -311,7 +358,6 @@ class ProtonDriver(base.BaseDriver):
             # Must log, and determine best way to communicate this failure
             # back up to the caller
             reply = unmarshal_response(reply, self._allowed_remote_exmods)
-        LOG.debug("Send to %s returning", target)
         return reply
 
     @_ensure_connect_called
@@ -333,21 +379,20 @@ class ProtonDriver(base.BaseDriver):
                       N means N retries
         :type retry: int
         """
-        request = marshal_request(message, ctxt, (version == 2.0))
+        request = marshal_request(message, ctxt, envelope=(version == 2.0))
         # no timeout is applied to notifications, however if the backend is
         # queueless this could lead to a hang - provide a default to prevent
         # this
         # TODO(kgiusti) should raise NotImplemented if not broker backend
-        LOG.debug("Send notification to %s", target)
         deadline = compute_timeout(self._default_notify_timeout)
+        ack = not self._pre_settle_notify
         task = controller.SendTask("Notify", request, target,
-                                   deadline, retry, wait_for_ack=True,
+                                   deadline, retry, wait_for_ack=ack,
                                    notification=True)
         self._ctrl.add_task(task)
         rc = task.wait()
         if isinstance(rc, Exception):
             raise rc
-        LOG.debug("Send notification to %s returning", target)
 
     @_ensure_connect_called
     def listen(self, target, batch_size, batch_timeout):
