@@ -38,6 +38,9 @@ class FakeIncomingMessage(base.RpcIncomingMessage):
     def requeue(self):
         self.requeue_callback()
 
+    def heartbeat(self):
+        """Heartbeat is not supported."""
+
 
 class FakeListener(base.PollStyleListener):
 
@@ -142,10 +145,11 @@ class FakeExchange(object):
 
 
 class FakeExchangeManager(object):
+    _exchanges_lock = threading.Lock()
+    _exchanges = {}
+
     def __init__(self, default_exchange):
         self._default_exchange = default_exchange
-        self._exchanges_lock = threading.Lock()
-        self._exchanges = {}
 
     def get_exchange(self, name):
         if name is None:
@@ -212,7 +216,7 @@ class FakeDriver(base.BaseDriver):
         return None
 
     def send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
-             retry=None):
+             call_monitor_timeout=None, retry=None):
         # NOTE(sileht): retry doesn't need to be implemented, the fake
         # transport always works
         return self._send(target, ctxt, message, wait_for_reply, timeout)
