@@ -32,6 +32,7 @@ import kombu.entity
 import kombu.messaging
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import eventletutils
 from oslo_utils import netutils
 import six
 from six.moves.urllib import parse
@@ -53,25 +54,29 @@ TCP_USER_TIMEOUT = 18
 
 
 rabbit_opts = [
-    cfg.StrOpt('kombu_ssl_version',
+    cfg.BoolOpt('ssl',
+                default=False,
+                deprecated_name='rabbit_use_ssl',
+                help='Connect over SSL.'),
+    cfg.StrOpt('ssl_version',
                default='',
-               deprecated_group='DEFAULT',
+               deprecated_name='kombu_ssl_version',
                help='SSL version to use (valid only if SSL enabled). '
                     'Valid values are TLSv1 and SSLv23. SSLv2, SSLv3, '
                     'TLSv1_1, and TLSv1_2 may be available on some '
                     'distributions.'
                ),
-    cfg.StrOpt('kombu_ssl_keyfile',
+    cfg.StrOpt('ssl_key_file',
                default='',
-               deprecated_group='DEFAULT',
+               deprecated_name='kombu_ssl_keyfile',
                help='SSL key file (valid only if SSL enabled).'),
-    cfg.StrOpt('kombu_ssl_certfile',
+    cfg.StrOpt('ssl_cert_file',
                default='',
-               deprecated_group='DEFAULT',
+               deprecated_name='kombu_ssl_certfile',
                help='SSL cert file (valid only if SSL enabled).'),
-    cfg.StrOpt('kombu_ssl_ca_certs',
+    cfg.StrOpt('ssl_ca_file',
                default='',
-               deprecated_group='DEFAULT',
+               deprecated_name='kombu_ssl_ca_certs',
                help='SSL certification authority file '
                     '(valid only if SSL enabled).'),
     cfg.FloatOpt('kombu_reconnect_delay',
@@ -115,10 +120,6 @@ rabbit_opts = [
                 deprecated_for_removal=True,
                 deprecated_reason="Replaced by [DEFAULT]/transport_url",
                 help='RabbitMQ HA cluster host:port pairs.'),
-    cfg.BoolOpt('rabbit_use_ssl',
-                default=False,
-                deprecated_group='DEFAULT',
-                help='Connect over SSL for RabbitMQ.'),
     cfg.StrOpt('rabbit_userid',
                default='guest',
                deprecated_group='DEFAULT',
@@ -133,6 +134,7 @@ rabbit_opts = [
                help='The RabbitMQ password.',
                secret=True),
     cfg.StrOpt('rabbit_login_method',
+               choices=('PLAIN', 'AMQPLAIN', 'RABBIT-CR-DEMO'),
                default='AMQPLAIN',
                deprecated_group='DEFAULT',
                help='The RabbitMQ login method.'),
@@ -168,7 +170,7 @@ rabbit_opts = [
                 'database. In RabbitMQ 3.0, queue mirroring is no longer '
                 'controlled by the x-ha-policy argument when declaring a '
                 'queue. If you just want to make sure that all queues (except '
-                ' those with auto-generated names) are mirrored across all '
+                'those with auto-generated names) are mirrored across all '
                 'nodes, run: '
                 """\"rabbitmqctl set_policy HA '^(?!amq\.).*' """
                 """'{"ha-mode": "all"}' \""""),
@@ -192,13 +194,6 @@ rabbit_opts = [
                default=2,
                help='How often times during the heartbeat_timeout_threshold '
                'we check the heartbeat.'),
-
-    # NOTE(sileht): deprecated option since oslo_messaging 1.5.0,
-    cfg.BoolOpt('fake_rabbit',
-                default=False,
-                deprecated_group='DEFAULT',
-                help='Deprecated, use rpc_backend=kombu+memory or '
-                'rpc_backend=fake'),
 ]
 
 LOG = logging.getLogger(__name__)
@@ -261,7 +256,7 @@ class Consumer(object):
     def __init__(self, exchange_name, queue_name, routing_key, type, durable,
                  exchange_auto_delete, queue_auto_delete, callback,
                  nowait=False, rabbit_ha_queues=None, rabbit_queue_ttl=0):
-        """Init the Publisher class with the exchange_name, routing_key,
+        """Init the Consumer class with the exchange_name, routing_key,
         type, durable auto_delete
         """
         self.queue_name = queue_name
@@ -275,8 +270,8 @@ class Consumer(object):
         self.nowait = nowait
         self.queue_arguments = _get_queue_arguments(rabbit_ha_queues,
                                                     rabbit_queue_ttl)
-
         self.queue = None
+        self._declared_on = None
         self.exchange = kombu.entity.Exchange(
             name=exchange_name,
             type=type,
@@ -285,6 +280,7 @@ class Consumer(object):
 
     def declare(self, conn):
         """Re-declare the queue after a rabbit (re)connect."""
+
         self.queue = kombu.entity.Queue(
             name=self.queue_name,
             channel=conn.channel,
@@ -308,17 +304,66 @@ class Consumer(object):
                 self.queue.declare()
             else:
                 raise
+        except kombu.exceptions.ConnectionError as exc:
+            # NOTE(gsantomaggio): This exception happens when the
+            # connection is established,but it fails to create the queue.
+            # Add some delay to avoid too many requests to the server.
+            # See: https://bugs.launchpad.net/oslo.messaging/+bug/1822778
+            # for details.
+            if exc.code == 541:
+                interval = 2
+                info = {'sleep_time': interval,
+                        'queue': self.queue_name,
+                        'err_str': exc
+                        }
+                LOG.error(_LE('Internal amqp error (541) '
+                              'during queue declare,'
+                              'retrying in %(sleep_time)s seconds. '
+                              'Queue: [%(queue)s], '
+                              'error message: [%(err_str)s]'), info)
+                time.sleep(interval)
+                self.queue.declare()
+            else:
+                raise
 
-    def consume(self, tag):
+        self._declared_on = conn.channel
+
+    def consume(self, conn, tag):
         """Actually declare the consumer on the amqp channel.  This will
         start the flow of messages from the queue.  Using the
         Connection.consume() will process the messages,
         calling the appropriate callback.
         """
 
-        self.queue.consume(callback=self._callback,
-                           consumer_tag=six.text_type(tag),
-                           nowait=self.nowait)
+        # Ensure we are on the correct channel before consuming
+        if conn.channel != self._declared_on:
+            self.declare(conn)
+        try:
+            self.queue.consume(callback=self._callback,
+                               consumer_tag=six.text_type(tag),
+                               nowait=self.nowait)
+        except conn.connection.channel_errors as exc:
+            # We retries once because of some races that we can
+            # recover before informing the deployer
+            # bugs.launchpad.net/oslo.messaging/+bug/1581148
+            # bugs.launchpad.net/oslo.messaging/+bug/1609766
+            # bugs.launchpad.net/neutron/+bug/1318721
+
+            # 406 error code relates to messages that are doubled ack'd
+
+            # At any channel error, the RabbitMQ closes
+            # the channel, but the amqp-lib quietly re-open
+            # it. So, we must reset all tags and declare
+            # all consumers again.
+            conn._new_tags = set(conn._consumers.values())
+            if exc.code == 404 or (exc.code == 406 and
+                                   exc.method_name == 'Basic.ack'):
+                self.declare(conn)
+                self.queue.consume(callback=self._callback,
+                                   consumer_tag=six.text_type(tag),
+                                   nowait=self.nowait)
+            else:
+                raise
 
     def cancel(self, tag):
         LOG.trace('ConsumerBase.cancel: canceling %s', tag)
@@ -338,7 +383,7 @@ class Consumer(object):
         except Exception:
             LOG.exception(_LE("Failed to process message"
                               " ... skipping it."))
-            message.ack()
+            message.reject()
 
 
 class DummyConnectionLock(_utils.DummyLock):
@@ -347,9 +392,9 @@ class DummyConnectionLock(_utils.DummyLock):
 
 
 class ConnectionLock(DummyConnectionLock):
-    """Lock object to protect access the the kombu connection
+    """Lock object to protect access to the kombu connection
 
-    This is a lock object to protect access the the kombu connection
+    This is a lock object to protect access to the kombu connection
     object between the heartbeat thread and the driver thread.
 
     They are two way to acquire this lock:
@@ -374,7 +419,7 @@ class ConnectionLock(DummyConnectionLock):
         self._monitor = threading.Lock()
         self._workers_locks = threading.Condition(self._monitor)
         self._heartbeat_lock = threading.Condition(self._monitor)
-        self._get_thread_id = _utils.fetch_current_thread_functor()
+        self._get_thread_id = eventletutils.fetch_current_thread_functor()
 
     def acquire(self):
         with self._monitor:
@@ -432,7 +477,6 @@ class Connection(object):
         self.interval_max = driver_conf.rabbit_interval_max
 
         self.login_method = driver_conf.rabbit_login_method
-        self.fake_rabbit = driver_conf.fake_rabbit
         self.virtual_host = driver_conf.rabbit_virtual_host
         self.rabbit_hosts = driver_conf.rabbit_hosts
         self.rabbit_port = driver_conf.rabbit_port
@@ -448,17 +492,17 @@ class Connection(object):
         self.kombu_reconnect_delay = driver_conf.kombu_reconnect_delay
         self.amqp_durable_queues = driver_conf.amqp_durable_queues
         self.amqp_auto_delete = driver_conf.amqp_auto_delete
-        self.rabbit_use_ssl = driver_conf.rabbit_use_ssl
+        self.ssl = driver_conf.ssl
         self.kombu_missing_consumer_retry_timeout = \
             driver_conf.kombu_missing_consumer_retry_timeout
         self.kombu_failover_strategy = driver_conf.kombu_failover_strategy
         self.kombu_compression = driver_conf.kombu_compression
 
-        if self.rabbit_use_ssl:
-            self.kombu_ssl_version = driver_conf.kombu_ssl_version
-            self.kombu_ssl_keyfile = driver_conf.kombu_ssl_keyfile
-            self.kombu_ssl_certfile = driver_conf.kombu_ssl_certfile
-            self.kombu_ssl_ca_certs = driver_conf.kombu_ssl_ca_certs
+        if self.ssl:
+            self.ssl_version = driver_conf.ssl_version
+            self.ssl_key_file = driver_conf.ssl_key_file
+            self.ssl_cert_file = driver_conf.ssl_cert_file
+            self.ssl_ca_file = driver_conf.ssl_ca_file
 
         # Try forever?
         if self.max_retries <= 0:
@@ -470,12 +514,7 @@ class Connection(object):
             virtual_host = self.virtual_host
 
         self._url = ''
-        if self.fake_rabbit:
-            LOG.warning(_LW("Deprecated: fake_rabbit option is deprecated, "
-                            "set rpc_backend to kombu+memory or use the fake "
-                            "driver instead."))
-            self._url = 'memory://%s/' % virtual_host
-        elif url.hosts:
+        if url.hosts:
             if url.transport.startswith('kombu+'):
                 LOG.warning(_LW('Selecting the kombu transport through the '
                                 'transport url (%s) is a experimental feature '
@@ -571,7 +610,7 @@ class Connection(object):
         # expected waiting the events drain, we start heartbeat_check and
         # retrieve the server heartbeat packet only two times more than
         # the minimum required for the heartbeat works
-        # (heatbeat_timeout/heartbeat_rate/2.0, default kombu
+        # (heartbeat_timeout/heartbeat_rate/2.0, default kombu
         # heartbeat_rate is 2)
         self._heartbeat_wait_timeout = (
             float(self.heartbeat_timeout_threshold) /
@@ -579,7 +618,8 @@ class Connection(object):
         self._heartbeat_support_log_emitted = False
 
         # NOTE(sileht): just ensure the connection is setuped at startup
-        self.ensure_connection()
+        with self._connection_lock:
+            self.ensure_connection()
 
         # NOTE(sileht): if purpose is PURPOSE_LISTEN
         # the consume code does the heartbeat stuff
@@ -595,7 +635,7 @@ class Connection(object):
 
         # NOTE(sileht): value chosen according the best practice from kombu
         # http://kombu.readthedocs.org/en/latest/reference/kombu.common.html#kombu.common.eventloop
-        # For heatbeat, we can set a bigger timeout, and check we receive the
+        # For heartbeat, we can set a bigger timeout, and check we receive the
         # heartbeat packets regulary
         if self._heartbeat_supported_and_enabled():
             self._poll_timeout = self._heartbeat_wait_timeout
@@ -647,19 +687,19 @@ class Connection(object):
         """Handles fetching what ssl params should be used for the connection
         (if any).
         """
-        if self.rabbit_use_ssl:
+        if self.ssl:
             ssl_params = dict()
 
             # http://docs.python.org/library/ssl.html - ssl.wrap_socket
-            if self.kombu_ssl_version:
+            if self.ssl_version:
                 ssl_params['ssl_version'] = self.validate_ssl_version(
-                    self.kombu_ssl_version)
-            if self.kombu_ssl_keyfile:
-                ssl_params['keyfile'] = self.kombu_ssl_keyfile
-            if self.kombu_ssl_certfile:
-                ssl_params['certfile'] = self.kombu_ssl_certfile
-            if self.kombu_ssl_ca_certs:
-                ssl_params['ca_certs'] = self.kombu_ssl_ca_certs
+                    self.ssl_version)
+            if self.ssl_key_file:
+                ssl_params['keyfile'] = self.ssl_key_file
+            if self.ssl_cert_file:
+                ssl_params['certfile'] = self.ssl_cert_file
+            if self.ssl_ca_file:
+                ssl_params['ca_certs'] = self.ssl_ca_file
                 # We might want to allow variations in the
                 # future with this?
                 ssl_params['cert_reqs'] = ssl.CERT_REQUIRED
@@ -678,7 +718,7 @@ class Connection(object):
         # NOTE(sileht): we reset the channel and ensure
         # the kombu underlying connection works
         self._set_current_channel(None)
-        self.ensure(method=lambda: self.connection.connection)
+        self.ensure(method=self.connection.connect)
         self.set_transport_socket_timeout()
 
     def ensure(self, method, retry=None,
@@ -697,8 +737,8 @@ class Connection(object):
         if self._initial_pid != current_pid:
             LOG.warning(_LW("Process forked after connection established! "
                             "This can result in unpredictable behavior. "
-                            "See: http://docs.openstack.org/developer/"
-                            "oslo.messaging/transport.html"))
+                            "See: https://docs.openstack.org/oslo.messaging/"
+                            "latest/reference/transport.html"))
             self._initial_pid = current_pid
 
         if retry is None:
@@ -718,7 +758,7 @@ class Connection(object):
                         else interval)
 
             info = {'err_str': exc, 'sleep_time': interval}
-            info.update(self._get_connection_info())
+            info.update(self._get_connection_info(conn_error=True))
 
             if 'Socket closed' in six.text_type(exc):
                 LOG.error(_LE('[%(connection_id)s] AMQP server'
@@ -729,8 +769,7 @@ class Connection(object):
                 LOG.error(_LE('[%(connection_id)s] AMQP server on '
                               '%(hostname)s:%(port)s is unreachable: '
                               '%(err_str)s. Trying again in '
-                              '%(sleep_time)d seconds. Client port: '
-                              '%(client_port)s'), info)
+                              '%(sleep_time)d seconds.'), info)
 
             # XXX(nic): when reconnecting to a RabbitMQ cluster
             # with mirrored queues in use, the attempt to release the
@@ -751,14 +790,11 @@ class Connection(object):
             """Callback invoked when the kombu reconnects and creates
             a new channel, we use it the reconfigure our consumers.
             """
-
-            self.set_transport_socket_timeout()
             self._set_current_channel(new_channel)
-            for consumer in self._consumers:
-                consumer.declare(self)
+            self.set_transport_socket_timeout()
 
             LOG.info(_LI('[%(connection_id)s] Reconnected to AMQP server on '
-                         '%(hostname)s:%(port)s via [%(transport)s] client'
+                         '%(hostname)s:%(port)s via [%(transport)s] client '
                          'with port %(client_port)s.'),
                      self._get_connection_info())
 
@@ -766,19 +802,6 @@ class Connection(object):
             self._set_current_channel(channel)
             method()
 
-        # NOTE(sileht): Some dummy driver like the in-memory one doesn't
-        # have notion of recoverable connection, so we must raise the original
-        # exception like kombu does in this case.
-        has_modern_errors = hasattr(
-            self.connection.transport, 'recoverable_connection_errors',
-        )
-        if has_modern_errors:
-            recoverable_errors = (
-                self.connection.recoverable_channel_errors +
-                self.connection.recoverable_connection_errors)
-        else:
-            recoverable_errors = ()
-
         try:
             autoretry_method = self.connection.autoretry(
                 execute_method, channel=self.channel,
@@ -791,9 +814,11 @@ class Connection(object):
             ret, channel = autoretry_method()
             self._set_current_channel(channel)
             return ret
-        except recoverable_errors as exc:
-            LOG.debug("Received recoverable error from kombu:",
-                      exc_info=True)
+        except rpc_amqp.AMQPDestinationNotFound:
+            # NOTE(sileht): we must reraise this without
+            # trigger error_callback
+            raise
+        except Exception as exc:
             error_callback and error_callback(exc)
             self._set_current_channel(None)
             # NOTE(sileht): number of retry exceeded and the connection
@@ -805,13 +830,6 @@ class Connection(object):
                     'tries: %(err_str)s') % info
             LOG.error(msg)
             raise exceptions.MessageDeliveryFailure(msg)
-        except rpc_amqp.AMQPDestinationNotFound:
-            # NOTE(sileht): we must reraise this without
-            # trigger error_callback
-            raise
-        except Exception as exc:
-            error_callback and error_callback(exc)
-            raise
 
     def _set_current_channel(self, new_channel):
         """Change the channel to use.
@@ -832,6 +850,8 @@ class Connection(object):
             if self.purpose == rpc_common.PURPOSE_LISTEN:
                 self._set_qos(new_channel)
             self._producer = kombu.messaging.Producer(new_channel)
+            for consumer in self._consumers:
+                consumer.declare(self)
 
     def _set_qos(self, channel):
         """Set QoS prefetch count on the channel"""
@@ -855,13 +875,11 @@ class Connection(object):
 
     def reset(self):
         """Reset a connection so it can be used again."""
-        recoverable_errors = (self.connection.recoverable_channel_errors +
-                              self.connection.recoverable_connection_errors)
         with self._connection_lock:
             try:
                 for consumer, tag in self._consumers.items():
                     consumer.cancel(tag=tag)
-            except recoverable_errors:
+            except kombu.exceptions.OperationalError:
                 self.ensure_connection()
             self._consumers.clear()
             self._active_tags.clear()
@@ -939,7 +957,7 @@ class Connection(object):
 
     def _heartbeat_start(self):
         if self._heartbeat_supported_and_enabled():
-            self._heartbeat_exit_event = threading.Event()
+            self._heartbeat_exit_event = eventletutils.Event()
             self._heartbeat_thread = threading.Thread(
                 target=self._heartbeat_thread_job)
             self._heartbeat_thread.daemon = True
@@ -956,13 +974,17 @@ class Connection(object):
     def _heartbeat_thread_job(self):
         """Thread that maintains inactive connections
         """
+        # NOTE(hberaud): Python2 doesn't have ConnectionRefusedError
+        # defined so to switch connections destination on failure
+        # with python2 and python3 we need to wrapp adapt connection refused
+        try:
+            ConnectRefuseError = ConnectionRefusedError
+        except NameError:
+            ConnectRefuseError = socket.error
+
         while not self._heartbeat_exit_event.is_set():
             with self._connection_lock.for_heartbeat():
 
-                recoverable_errors = (
-                    self.connection.recoverable_channel_errors +
-                    self.connection.recoverable_connection_errors)
-
                 try:
                     try:
                         self._heartbeat_check()
@@ -976,13 +998,24 @@ class Connection(object):
                             self.connection.drain_events(timeout=0.001)
                         except socket.timeout:
                             pass
-                    except recoverable_errors as exc:
+                    # NOTE(hberaud): In a clustered rabbitmq when
+                    # a node disappears, we get a ConnectionRefusedError
+                    # because the socket get disconnected.
+                    # The socket access yields a OSError because the heartbeat
+                    # tries to reach an unreachable host (No route to host).
+                    # Catch these exceptions to ensure that we call
+                    # ensure_connection for switching the
+                    # connection destination.
+                    except (socket.timeout,
+                            ConnectRefuseError,
+                            OSError,
+                            kombu.exceptions.OperationalError) as exc:
                         LOG.info(_LI("A recoverable connection/channel error "
                                      "occurred, trying to reconnect: %s"), exc)
                         self.ensure_connection()
                 except Exception:
-                    LOG.warning(_LW("Unexpected error during heartbeart "
-                                    "thread processing, retrying..."))
+                    LOG.warning(_LW("Unexpected error during heartbeat "
+                                "thread processing, retrying..."))
                     LOG.debug('Exception', exc_info=True)
 
             self._heartbeat_exit_event.wait(
@@ -1020,14 +1053,13 @@ class Connection(object):
         timer = rpc_common.DecayingTimer(duration=timeout)
         timer.start()
 
-        def _raise_timeout(exc):
-            LOG.debug('Timed out waiting for RPC response: %s', exc)
+        def _raise_timeout():
             raise rpc_common.Timeout()
 
         def _recoverable_error_callback(exc):
             if not isinstance(exc, rpc_common.Timeout):
                 self._new_tags = set(self._consumers.values())
-            timer.check_return(_raise_timeout, exc)
+            timer.check_return(_raise_timeout)
 
         def _error_callback(exc):
             _recoverable_error_callback(exc)
@@ -1042,31 +1074,11 @@ class Connection(object):
             if not self.connection.connected:
                 raise self.connection.recoverable_connection_errors[0]
 
-            consume_max_retries = 2
-            while self._new_tags and consume_max_retries:
+            while self._new_tags:
                 for consumer, tag in self._consumers.items():
                     if tag in self._new_tags:
-                        try:
-                            consumer.consume(tag=tag)
-                            self._new_tags.remove(tag)
-                        except self.connection.channel_errors as exc:
-                            # NOTE(kbespalov): during the interval between
-                            # a queue declaration and consumer declaration
-                            # the queue can disappear. In this case
-                            # we must redeclare queue and try to re-consume.
-                            # More details is here:
-                            # bugs.launchpad.net/oslo.messaging/+bug/1581148
-                            if exc.code == 404 and consume_max_retries:
-                                consumer.declare(self)
-                                # NOTE(kbespalov): the broker closes a channel
-                                # at any channel error. The py-amqp catches
-                                # this situation and re-open a new channel.
-                                # So, we must re-declare all consumers again.
-                                self._new_tags = set(self._consumers.values())
-                                consume_max_retries -= 1
-                                break
-                            else:
-                                raise
+                        consumer.consume(self, tag=tag)
+                        self._new_tags.remove(tag)
 
             poll_timeout = (self._poll_timeout if timeout is None
                             else min(timeout, self._poll_timeout))
@@ -1080,9 +1092,15 @@ class Connection(object):
                 try:
                     self.connection.drain_events(timeout=poll_timeout)
                     return
-                except socket.timeout as exc:
+                except socket.timeout:
                     poll_timeout = timer.check_return(
-                        _raise_timeout, exc, maximum=self._poll_timeout)
+                        _raise_timeout, maximum=self._poll_timeout)
+                except self.connection.channel_errors as exc:
+                    if exc.code == 406 and exc.method_name == 'Basic.ack':
+                        # NOTE(gordc): occasionally multiple workers will grab
+                        # same message and acknowledge it. if it happens, meh.
+                        raise self.connection.recoverable_channel_errors[0]
+                    raise
 
         with self._connection_lock:
             self.ensure(_consume,
@@ -1161,10 +1179,18 @@ class Connection(object):
         with self._connection_lock:
             self.ensure(method, retry=retry, error_callback=_error_callback)
 
-    def _get_connection_info(self):
-        info = self.connection.info()
+    def _get_connection_info(self, conn_error=False):
+        # Bug #1745166: set 'conn_error' true if this is being called when the
+        # connection is in a known error state.  Otherwise attempting to access
+        # the connection's socket while it is in an error state will cause
+        # py-amqp to attempt reconnecting.
+        ci = self.connection.info()
+        info = dict([(k, ci.get(k)) for k in
+                     ['hostname', 'port', 'transport']])
         client_port = None
-        if self.channel and hasattr(self.channel.connection, 'sock'):
+        if (not conn_error and self.channel
+                and hasattr(self.channel.connection, 'sock')
+                and self.channel.connection.sock):
             client_port = self.channel.connection.sock.getsockname()[1]
         info.update({'client_port': client_port,
                      'connection_id': self.connection_id})
@@ -1318,6 +1344,7 @@ class RabbitDriver(amqpdriver.AMQPDriverBase):
         conf.register_opts(rabbit_opts, group=opt_group)
         conf.register_opts(rpc_amqp.amqp_opts, group=opt_group)
         conf.register_opts(base.base_opts, group=opt_group)
+        conf = rpc_common.ConfigOptsProxy(conf, url, opt_group.name)
 
         self.missing_destination_retry_timeout = (
             conf.oslo_messaging_rabbit.kombu_missing_consumer_retry_timeout)
