@@ -27,34 +27,18 @@ def get_tcp_random_address(conf):
     return "tcp://%s" % conf.oslo_messaging_zmq.rpc_zmq_bind_address
 
 
-def get_broker_address(conf):
-    return "ipc://%s/zmq-broker" % conf.oslo_messaging_zmq.rpc_zmq_ipc_dir
-
-
 def prefix_str(key, listener_type):
-    return listener_type + "_" + key
+    return listener_type + "/" + key
 
 
 def target_to_key(target, listener_type=None):
-
-    def prefix(key):
-        return prefix_str(key, listener_type) if listener_type is not None \
-            else key
-
-    if target.topic and target.server:
-        attributes = ['topic', 'server']
-        key = ".".join(getattr(target, attr) for attr in attributes)
-        return prefix(key)
-    if target.topic:
-        return prefix(target.topic)
+    key = target.topic
+    if target.server and not target.fanout:
+        # FIXME(ozamiatin): Workaround for Cinder.
+        # Remove split when Bug #1630975 is being fixed.
+        key += "/" + target.server.split('@')[0]
+    return prefix_str(key, listener_type) if listener_type else key
 
 
 def target_to_subscribe_filter(target):
-    if target.topic and target.server:
-        attributes = ['topic', 'server']
-        key = "/".join(getattr(target, attr) for attr in attributes)
-        return six.b(key)
-    if target.topic:
-        return six.b(target.topic)
-    if target.server:
-        return six.b(target.server)
+    return six.b(target.topic)
