@@ -20,7 +20,6 @@ import argparse
 import logging
 import uuid
 
-from debtcollector import renames
 from oslo_config import cfg
 from oslo_utils import timeutils
 import six
@@ -55,6 +54,10 @@ _notifier_opts = [
                                       group='DEFAULT')
                 ],
                 help='AMQP topic used for OpenStack notifications.'),
+    cfg.IntOpt('retry', default=-1,
+               help='The maximum number of attempts to re-send a notification '
+                    'message which failed to be delivered due to a '
+                    'recoverable error. 0 - No retry, -1 - indefinite'),
 ]
 
 _LOG = logging.getLogger(__name__)
@@ -127,17 +130,17 @@ class Driver(object):
         :type msg: str
         :param priority: priority of the message
         :type priority: str
-        :param retry: an connection retries configuration
-                      None or -1 means to retry forever
-                      0 means no retry
-                      N means N retries
+        :param retry: connection retries configuration (used by the messaging
+                      driver):
+                      None or -1 means to retry forever.
+                      0 means no retry is attempted.
+                      N means attempt at most N retries.
         :type retry: int
         """
         pass
 
 
-def get_notification_transport(conf, url=None,
-                               allowed_remote_exmods=None, aliases=None):
+def get_notification_transport(conf, url=None, allowed_remote_exmods=None):
     """A factory method for Transport objects for notifications.
 
     This method should be used for notifications, in case notifications are
@@ -154,21 +157,20 @@ def get_notification_transport(conf, url=None,
 
     :param conf: the user configuration
     :type conf: cfg.ConfigOpts
-    :param url: a transport URL
+    :param url: a transport URL, see :py:class:`transport.TransportURL`
     :type url: str or TransportURL
     :param allowed_remote_exmods: a list of modules which a client using this
                                   transport will deserialize remote exceptions
                                   from
     :type allowed_remote_exmods: list
-    :param aliases: A map of transport alias to transport name
-    :type aliases: dict
     """
     conf.register_opts(_notifier_opts,
                        group='oslo_messaging_notifications')
     if url is None:
         url = conf.oslo_messaging_notifications.transport_url
-    return msg_transport.get_transport(conf, url,
-                                       allowed_remote_exmods, aliases)
+    return msg_transport._get_transport(
+        conf, url, allowed_remote_exmods,
+        transport_cls=msg_transport.NotificationTransport)
 
 
 class Notifier(object):
@@ -204,7 +206,7 @@ class Notifier(object):
         notifier = notifier.Notifier(transport,
                                      'compute.host',
                                      driver='messaging',
-                                     topic='notifications')
+                                     topics=['notifications'])
 
     Notifier objects are relatively expensive to instantiate (mostly the cost
     of loading notification drivers), so it is possible to specialize a given
@@ -214,13 +216,8 @@ class Notifier(object):
         notifier.info(ctxt, event_type, payload)
     """
 
-    @renames.renamed_kwarg('topic', 'topics',
-                           message="Please use topics instead of topic",
-                           version='4.5.0',
-                           removal_version='5.0.0')
     def __init__(self, transport, publisher_id=None,
-                 driver=None, topic=None,
-                 serializer=None, retry=None,
+                 driver=None, serializer=None, retry=None,
                  topics=None):
         """Construct a Notifier object.
 
@@ -231,14 +228,13 @@ class Notifier(object):
         :type publisher_id: str
         :param driver: a driver to lookup from oslo_messaging.notify.drivers
         :type driver: str
-        :param topic: the topic which to send messages on
-        :type topic: str
         :param serializer: an optional entity serializer
         :type serializer: Serializer
-        :param retry: an connection retries configuration
-                      None or -1 means to retry forever
-                      0 means no retry
-                      N means N retries
+        :param retry: connection retries configuration (used by the messaging
+                      driver):
+                      None or -1 means to retry forever.
+                      0 means no retry is attempted.
+                      N means attempt at most N retries.
         :type retry: int
         :param topics: the topics which to send messages on
         :type topics: list of strings
@@ -247,17 +243,22 @@ class Notifier(object):
         conf.register_opts(_notifier_opts,
                            group='oslo_messaging_notifications')
 
+        if not isinstance(transport, msg_transport.NotificationTransport):
+            _LOG.warning("Using RPC transport for notifications. Please use "
+                         "get_notification_transport to obtain a "
+                         "notification transport instance.")
         self.transport = transport
         self.publisher_id = publisher_id
-        self.retry = retry
+        if retry is not None:
+            self.retry = retry
+        else:
+            self.retry = conf.oslo_messaging_notifications.retry
 
         self._driver_names = ([driver] if driver is not None else
                               conf.oslo_messaging_notifications.driver)
 
         if topics is not None:
             self._topics = topics
-        elif topic is not None:
-            self._topics = [topic]
         else:
             self._topics = conf.oslo_messaging_notifications.topics
         self._serializer = serializer or msg_serializer.NoOpSerializer()
@@ -285,10 +286,11 @@ class Notifier(object):
         :param publisher_id: field in notifications sent, for example
                              'compute.host1'
         :type publisher_id: str
-        :param retry: an connection retries configuration
-                      None or -1 means to retry forever
-                      0 means no retry
-                      N means N retries
+        :param retry: connection retries configuration (used by the messaging
+                      driver):
+                      None or -1 means to retry forever.
+                      0 means no retry is attempted.
+                      N means attempt at most N retries.
         :type retry: int
         """
         return _SubNotifier._prepare(self, publisher_id, retry=retry)
@@ -422,6 +424,14 @@ class Notifier(object):
         """
         self._notify(ctxt, event_type, payload, 'SAMPLE')
 
+    def is_enabled(self):
+        """Check if the notifier will emit notifications anywhere.
+
+        :return: false if the driver of the notifier is set only to noop, true
+                 otherwise
+        """
+        return self._driver_mgr.names() != ['noop']
+
 
 class _SubNotifier(Notifier):
 
