@@ -24,6 +24,7 @@ __all__ = [
 ]
 
 import abc
+import logging
 
 from oslo_config import cfg
 import six
@@ -32,6 +33,10 @@ from oslo_messaging._drivers import base as driver_base
 from oslo_messaging import _utils as utils
 from oslo_messaging import exceptions
 from oslo_messaging import serializer as msg_serializer
+from oslo_messaging import transport as msg_transport
+
+
+LOG = logging.getLogger(__name__)
 
 _client_opts = [
     cfg.IntOpt('rpc_response_timeout',
@@ -89,13 +94,15 @@ class _BaseCallContext(object):
     _marker = object()
 
     def __init__(self, transport, target, serializer,
-                 timeout=None, version_cap=None, retry=None):
+                 timeout=None, version_cap=None, retry=None,
+                 call_monitor_timeout=None):
         self.conf = transport.conf
 
         self.transport = transport
         self.target = target
         self.serializer = serializer
         self.timeout = timeout
+        self.call_monitor_timeout = call_monitor_timeout
         self.retry = retry
         self.version_cap = version_cap
 
@@ -105,7 +112,7 @@ class _BaseCallContext(object):
         msg = dict(method=method)
 
         msg['args'] = dict()
-        for argname, arg in six.iteritems(args):
+        for argname, arg in args.items():
             msg['args'][argname] = self.serializer.serialize_entity(ctxt, arg)
 
         if self.target.namespace is not None:
@@ -161,11 +168,14 @@ class _BaseCallContext(object):
         if self.timeout is None:
             timeout = self.conf.rpc_response_timeout
 
+        cm_timeout = self.call_monitor_timeout
+
         self._check_version_cap(msg.get('version'))
 
         try:
             result = self.transport._send(self.target, msg_ctxt, msg,
                                           wait_for_reply=True, timeout=timeout,
+                                          call_monitor_timeout=cm_timeout,
                                           retry=self.retry)
         except driver_base.TransportDriverError as ex:
             raise ClientSendError(self.target, ex)
@@ -175,7 +185,8 @@ class _BaseCallContext(object):
     @abc.abstractmethod
     def prepare(self, exchange=_marker, topic=_marker, namespace=_marker,
                 version=_marker, server=_marker, fanout=_marker,
-                timeout=_marker, version_cap=_marker, retry=_marker):
+                timeout=_marker, version_cap=_marker, retry=_marker,
+                call_monitor_timeout=_marker):
         """Prepare a method invocation context. See RPCClient.prepare()."""
 
 
@@ -187,7 +198,8 @@ class _CallContext(_BaseCallContext):
     def _prepare(cls, call_context,
                  exchange=_marker, topic=_marker, namespace=_marker,
                  version=_marker, server=_marker, fanout=_marker,
-                 timeout=_marker, version_cap=_marker, retry=_marker):
+                 timeout=_marker, version_cap=_marker, retry=_marker,
+                 call_monitor_timeout=_marker):
         cls._check_version(version)
         kwargs = dict(
             exchange=exchange,
@@ -206,35 +218,55 @@ class _CallContext(_BaseCallContext):
             version_cap = call_context.version_cap
         if retry is cls._marker:
             retry = call_context.retry
+        if call_monitor_timeout is cls._marker:
+            call_monitor_timeout = call_context.call_monitor_timeout
 
         return _CallContext(call_context.transport, target,
                             call_context.serializer,
-                            timeout, version_cap, retry)
+                            timeout, version_cap, retry,
+                            call_monitor_timeout)
 
     def prepare(self, exchange=_marker, topic=_marker, namespace=_marker,
                 version=_marker, server=_marker, fanout=_marker,
-                timeout=_marker, version_cap=_marker, retry=_marker):
+                timeout=_marker, version_cap=_marker, retry=_marker,
+                call_monitor_timeout=_marker):
         return _CallContext._prepare(self,
                                      exchange, topic, namespace,
                                      version, server, fanout,
-                                     timeout, version_cap, retry)
+                                     timeout, version_cap, retry,
+                                     call_monitor_timeout)
 
 
 class RPCClient(_BaseCallContext):
 
-    """A class for invoking methods on remote servers.
+    """A class for invoking methods on remote RPC servers.
 
-    The RPCClient class is responsible for sending method invocations to remote
-    servers via a messaging transport.
+    The RPCClient class is responsible for sending method invocations to and
+    receiving return values from remote RPC servers via a messaging transport.
 
-    A default target is supplied to the RPCClient constructor, but target
-    attributes can be overridden for individual method invocations using the
-    prepare() method.
+    Two RPC patterns are supported: RPC calls and RPC casts.
+
+    An RPC cast is used when an RPC method does *not* return a value to
+    the caller. An RPC call is used when a return value is expected from the
+    method.  For further information see the cast() and call() methods.
+
+    The default target used for all subsequent calls and casts is supplied to
+    the RPCClient constructor.  The client uses the target to control how the
+    RPC request is delivered to a server.  If only the target's topic (and
+    optionally exchange) are set, then the RPC can be serviced by any server
+    that is listening to that topic (and exchange).  If multiple servers are
+    listening on that topic/exchange, then one server is picked using a
+    best-effort round-robin algorithm.  Alternatively, the client can set the
+    Target's ``server`` attribute to the name of a specific server to send the
+    RPC request to one particular server.  In the case of RPC cast, the RPC
+    request can be broadcast to all servers listening to the Target's
+    topic/exchange by setting the Target's ``fanout`` property to ``True``.
+
+    While the default target is set on construction, target attributes can be
+    overridden for individual method invocations using the prepare() method.
 
     A method invocation consists of a request context dictionary, a method name
-    and a dictionary of arguments. A cast() invocation just sends the request
-    and returns immediately. A call() invocation waits for the server to send
-    a return value.
+    and a dictionary of arguments.
 
     This class is intended to be used by wrapping it in another class which
     provides methods on the subclass to perform the remote invocation using
@@ -267,7 +299,7 @@ class RPCClient(_BaseCallContext):
     However, this class can be used directly without wrapping it another class.
     For example::
 
-        transport = messaging.get_transport(cfg.CONF)
+        transport = messaging.get_rpc_transport(cfg.CONF)
         target = messaging.Target(topic='test', version='2.0')
         client = messaging.RPCClient(transport, target)
         client.call(ctxt, 'test', arg=arg)
@@ -275,10 +307,13 @@ class RPCClient(_BaseCallContext):
     but this is probably only useful in limited circumstances as a wrapper
     class will usually help to make the code much more obvious.
 
-    By default, cast() and call() will block until the message is successfully
-    sent. However, the retry parameter can be used to have message sending
-    fail with a MessageDeliveryFailure after the given number of retries. For
-    example::
+    If the connection to the messaging service is not active when an RPC
+    request is made the client will block waiting for the connection to
+    complete.  If the connection fails to complete, the client will try to
+    re-establish that connection. By default this will continue indefinitely
+    until the connection completes. However, the retry parameter can be used to
+    have the RPC request fail with a MessageDeliveryFailure after the given
+    number of retries. For example::
 
         client = messaging.RPCClient(transport, target, retry=None)
         client.call(ctxt, 'sync')
@@ -291,7 +326,8 @@ class RPCClient(_BaseCallContext):
     _marker = _BaseCallContext._marker
 
     def __init__(self, transport, target,
-                 timeout=None, version_cap=None, serializer=None, retry=None):
+                 timeout=None, version_cap=None, serializer=None, retry=None,
+                 call_monitor_timeout=None):
         """Construct an RPC client.
 
         :param transport: a messaging transport handle
@@ -304,24 +340,38 @@ class RPCClient(_BaseCallContext):
         :type version_cap: str
         :param serializer: an optional entity serializer
         :type serializer: Serializer
-        :param retry: an optional default connection retries configuration
-                      None or -1 means to retry forever
-                      0 means no retry
-                      N means N retries
+        :param retry: an optional default connection retries configuration:
+                      None or -1 means to retry forever.
+                      0 means no retry is attempted.
+                      N means attempt at most N retries.
         :type retry: int
+        :param call_monitor_timeout: an optional timeout (in seconds) for
+                                     active call heartbeating. If specified,
+                                     requires the server to heartbeat
+                                     long-running calls at this interval
+                                     (less than the overall timeout
+                                     parameter).
+        :type call_monitor_timeout: int
         """
         if serializer is None:
             serializer = msg_serializer.NoOpSerializer()
 
+        if not isinstance(transport, msg_transport.RPCTransport):
+            LOG.warning("Using notification transport for RPC. Please use "
+                        "get_rpc_transport to obtain an RPC transport "
+                        "instance.")
+
         super(RPCClient, self).__init__(
-            transport, target, serializer, timeout, version_cap, retry
+            transport, target, serializer, timeout, version_cap, retry,
+            call_monitor_timeout
         )
 
         self.conf.register_opts(_client_opts)
 
     def prepare(self, exchange=_marker, topic=_marker, namespace=_marker,
                 version=_marker, server=_marker, fanout=_marker,
-                timeout=_marker, version_cap=_marker, retry=_marker):
+                timeout=_marker, version_cap=_marker, retry=_marker,
+                call_monitor_timeout=_marker):
         """Prepare a method invocation context.
 
         Use this method to override client properties for an individual method
@@ -347,19 +397,42 @@ class RPCClient(_BaseCallContext):
         :type timeout: int or float
         :param version_cap: raise a RPCVersionCapError version exceeds this cap
         :type version_cap: str
-        :param retry: an optional connection retries configuration
-                      None or -1 means to retry forever
-                      0 means no retry
-                      N means N retries
+        :param retry: an optional connection retries configuration:
+                      None or -1 means to retry forever.
+                      0 means no retry is attempted.
+                      N means attempt at most N retries.
         :type retry: int
+        :param call_monitor_timeout: an optional timeout (in seconds) for
+                                     active call heartbeating. If specified,
+                                     requires the server to heartbeat
+                                     long-running calls at this interval
+                                     (less than the overall timeout
+                                     parameter).
+        :type call_monitor_timeout: int
         """
         return _CallContext._prepare(self,
                                      exchange, topic, namespace,
                                      version, server, fanout,
-                                     timeout, version_cap, retry)
+                                     timeout, version_cap, retry,
+                                     call_monitor_timeout)
 
     def cast(self, ctxt, method, **kwargs):
-        """Invoke a method and return immediately.
+        """Invoke a method without blocking for a return value.
+
+        The cast() method is used to invoke an RPC method that does not return
+        a value.  cast() RPC requests may be broadcast to all Servers listening
+        on a given topic by setting the fanout Target property to ``True``.
+
+        The cast() operation is best-effort: cast() will block the
+        calling thread until the RPC request method is accepted by the
+        messaging transport, but cast() does *not* verify that the RPC method
+        has been invoked by the server. cast() does guarantee that the
+        method will be not executed twice on a destination (e.g. 'at-most-once'
+        execution).
+
+        There are no ordering guarantees across successive casts, even
+        among casts to the same destination. Therefore methods may be executed
+        in an order different from the order in which they are cast.
 
         Method arguments must either be primitive types or types supported by
         the client's serializer (if any).
@@ -367,27 +440,37 @@ class RPCClient(_BaseCallContext):
         Similarly, the request context must be a dict unless the client's
         serializer supports serializing another type.
 
-        Note: cast does not ensure that the remote method will be executed on
-        each destination. But it does ensure that the method will be not
-        executed twice on a destination (e.g. 'at-most-once' execution).
-
-        Note: there are no ordering guarantees across successive casts, even
-        among casts to the same destination. Therefore methods may be executed
-        in an order different from the order in which they are cast.
-
         :param ctxt: a request context dict
         :type ctxt: dict
         :param method: the method name
         :type method: str
         :param kwargs: a dict of method arguments
         :type kwargs: dict
-        :raises: MessageDeliveryFailure
+        :raises: MessageDeliveryFailure if the messaging transport fails to
+                 accept the request.
+
         """
         self.prepare().cast(ctxt, method, **kwargs)
 
     def call(self, ctxt, method, **kwargs):
         """Invoke a method and wait for a reply.
 
+        The call() method is used to invoke RPC methods that return a
+        value. Since only a single return value is permitted it is not possible
+        to call() to a fanout target.
+
+        call() will block the calling thread until the messaging transport
+        provides the return value, a timeout occurs, or a non-recoverable error
+        occurs.
+
+        call() guarantees that the RPC request is done 'at-most-once' which
+        ensures that the call will never be duplicated.  However if the call
+        should fail or time out before the return value arrives then there are
+        no guarantees whether or not the method was invoked.
+
+        Since call() blocks until completion of the RPC method, call()s from
+        the same thread are guaranteed to be processed in-order.
+
         Method arguments must either be primitive types or types supported by
         the client's serializer (if any). Similarly, the request context must
         be a dict unless the client's serializer supports serializing another
@@ -397,12 +480,12 @@ class RPCClient(_BaseCallContext):
         method are handled are quite subtle.
 
         Firstly, if the remote exception is contained in one of the modules
-        listed in the allow_remote_exmods messaging.get_transport() parameter,
-        then it this exception will be re-raised by call(). However, such
-        locally re-raised remote exceptions are distinguishable from the same
-        exception type raised locally because re-raised remote exceptions are
-        modified such that their class name ends with the '_Remote' suffix so
-        you may do::
+        listed in the allow_remote_exmods messaging.get_rpc_transport()
+        parameter, then it this exception will be re-raised by call(). However,
+        such locally re-raised remote exceptions are distinguishable from the
+        same exception type raised locally because re-raised remote exceptions
+        are modified such that their class name ends with the '_Remote' suffix
+        so you may do::
 
             if ex.__class__.__name__.endswith('_Remote'):
                 # Some special case for locally re-raised remote exceptions
@@ -411,12 +494,6 @@ class RPCClient(_BaseCallContext):
         allowed_remote_exmods list, then a messaging.RemoteError exception is
         raised with all details of the remote exception.
 
-        Note: call is done 'at-most-once'. In case of we can't known
-        if the call have been done correctly, because we didn't get the
-        response on time, MessagingTimeout exception is raised.
-        The real reason can vary, transport failure, worker
-        doesn't answer in time or crash, ...
-
         :param ctxt: a request context dict
         :type ctxt: dict
         :param method: the method name
