@@ -20,28 +20,66 @@ methods which may be invoked remotely by clients over a given transport.
 To create an RPC server, you supply a transport, target and a list of
 endpoints.
 
-A transport can be obtained simply by calling the get_transport() method::
+A transport can be obtained simply by calling the get_rpc_transport() method::
 
-    transport = messaging.get_transport(conf)
+    transport = messaging.get_rpc_transport(conf)
 
 which will load the appropriate transport driver according to the user's
-messaging configuration. See get_transport() for more details.
+messaging configuration. See get_rpc_transport() for more details.
 
 The target supplied when creating an RPC server expresses the topic, server
 name and - optionally - the exchange to listen on. See Target for more details
 on these attributes.
 
+Multiple RPC Servers may listen to the same topic (and exchange)
+simultaneously. See RPCClient for details regarding how RPC requests are
+distributed to the Servers in this case.
+
 Each endpoint object may have a target attribute which may have namespace and
 version fields set. By default, we use the 'null namespace' and version 1.0.
 Incoming method calls will be dispatched to the first endpoint with the
 requested method, a matching namespace and a compatible version number.
 
-RPC servers have start(), stop() and wait() messages to begin handling
-requests, stop handling requests and wait for all in-process requests to
-complete.
+The first parameter to method invocations is always the request context
+supplied by the client.  The remaining parameters are the arguments supplied to
+the method by the client.  Endpoint methods may return a value.  If so the RPC
+Server will send the returned value back to the requesting client via the
+transport.
+
+The executor parameter controls how incoming messages will be received and
+dispatched. Refer to the Executor documentation for descriptions of the types
+of executors.
+
+*Note:* If the "eventlet" executor is used, the threading and time library need
+to be monkeypatched.
+
+The RPC reply operation is best-effort: the server will consider the message
+containing the reply successfully sent once it is accepted by the messaging
+transport.  The server does not guarantee that the reply is processed by the
+RPC client.  If the send fails an error will be logged and the server will
+continue to processing incoming RPC requests.
+
+Parameters to the method invocation and values returned from the method are
+python primitive types. However the actual encoding of the data in the message
+may not be in primitive form (e.g. the message payload may be a dictionary
+encoded as an ASCII string using JSON). A serializer object is used to convert
+incoming encoded message data to primitive types.  The serializer is also used
+to convert the return value from primitive types to an encoding suitable for
+the message payload.
+
+RPC servers have start(), stop() and wait() methods to begin handling
+requests, stop handling requests, and wait for all in-process requests to
+complete after the Server has been stopped.
 
 A simple example of an RPC server with multiple endpoints might be::
 
+    # NOTE(changzhi): We are using eventlet executor and
+    # time.sleep(1), therefore, the server code needs to be
+    # monkey-patched.
+
+    import eventlet
+    eventlet.monkey_patch()
+
     from oslo_config import cfg
     import oslo_messaging
     import time
@@ -63,14 +101,14 @@ A simple example of an RPC server with multiple endpoints might be::
         def test(self, ctx, arg):
             return arg
 
-    transport = oslo_messaging.get_transport(cfg.CONF)
+    transport = oslo_messaging.get_rpc_transport(cfg.CONF)
     target = oslo_messaging.Target(topic='test', server='server1')
     endpoints = [
         ServerControlEndpoint(None),
         TestEndpoint(),
     ]
     server = oslo_messaging.get_rpc_server(transport, target, endpoints,
-                                           executor='blocking')
+                                           executor='eventlet')
     try:
         server.start()
         while True:
@@ -81,25 +119,12 @@ A simple example of an RPC server with multiple endpoints might be::
     server.stop()
     server.wait()
 
-Clients can invoke methods on the server by sending the request to a topic and
-it gets sent to one of the servers listening on the topic, or by sending the
-request to a specific server listening on the topic, or by sending the request
-to all servers listening on the topic (known as fanout). These modes are chosen
-via the server and fanout attributes on Target but the mode used is transparent
-to the server.
-
-The first parameter to method invocations is always the request context
-supplied by the client.
-
-Parameters to the method invocation are primitive types and so must be the
-return values from the methods. By supplying a serializer object, a server can
-deserialize a request context and arguments from - and serialize return values
-to - primitive types.
 """
 
 __all__ = [
     'get_rpc_server',
     'expected_exceptions',
+    'expose'
 ]
 
 import logging
@@ -108,6 +133,7 @@ import sys
 from oslo_messaging._i18n import _LE
 from oslo_messaging.rpc import dispatcher as rpc_dispatcher
 from oslo_messaging import server as msg_server
+from oslo_messaging import transport as msg_transport
 
 LOG = logging.getLogger(__name__)
 
@@ -115,6 +141,10 @@ LOG = logging.getLogger(__name__)
 class RPCServer(msg_server.MessageHandlingServer):
     def __init__(self, transport, target, dispatcher, executor='blocking'):
         super(RPCServer, self).__init__(transport, dispatcher, executor)
+        if not isinstance(transport, msg_transport.RPCTransport):
+            LOG.warning("Using notification transport for RPC. Please use "
+                        "get_rpc_transport to obtain an RPC transport "
+                        "instance.")
         self._target = target
 
     def _create_listener(self):
@@ -122,6 +152,9 @@ class RPCServer(msg_server.MessageHandlingServer):
 
     def _process_incoming(self, incoming):
         message = incoming[0]
+
+        # TODO(sileht): We should remove that at some point and do
+        # this directly in the driver
         try:
             message.acknowledge()
         except Exception:
@@ -135,7 +168,7 @@ class RPCServer(msg_server.MessageHandlingServer):
             failure = e.exc_info
             LOG.debug(u'Expected exception during message handling (%s)', e)
         except Exception:
-            # current sys.exc_info() content can be overriden
+            # current sys.exc_info() content can be overridden
             # by another exception raised by a log handler during
             # LOG.exception(). So keep a copy and delete it later.
             failure = sys.exc_info()
@@ -156,29 +189,26 @@ class RPCServer(msg_server.MessageHandlingServer):
 
 
 def get_rpc_server(transport, target, endpoints,
-                   executor='blocking', serializer=None):
+                   executor='blocking', serializer=None, access_policy=None):
     """Construct an RPC server.
 
-    The executor parameter controls how incoming messages will be received and
-    dispatched. By default, the most simple executor is used - the blocking
-    executor.
-
-    If the eventlet executor is used, the threading and time library need to be
-    monkeypatched.
-
     :param transport: the messaging transport
     :type transport: Transport
     :param target: the exchange, topic and server to listen on
     :type target: Target
     :param endpoints: a list of endpoint objects
     :type endpoints: list
-    :param executor: name of a message executor - for example
-                     'eventlet', 'blocking'
+    :param executor: name of message executor - available values are
+                     'eventlet' and 'threading'
     :type executor: str
     :param serializer: an optional entity serializer
     :type serializer: Serializer
+    :param access_policy: an optional access policy.
+           Defaults to DefaultRPCAccessPolicy
+    :type access_policy: RPCAccessPolicyBase
     """
-    dispatcher = rpc_dispatcher.RPCDispatcher(endpoints, serializer)
+    dispatcher = rpc_dispatcher.RPCDispatcher(endpoints, serializer,
+                                              access_policy)
     return RPCServer(transport, target, dispatcher, executor)
 
 
@@ -207,3 +237,25 @@ def expected_exceptions(*exceptions):
                 raise rpc_dispatcher.ExpectedException()
         return inner
     return outer
+
+
+def expose(func):
+    """Decorator for RPC endpoint methods that are exposed to the RPC client.
+
+    If the dispatcher's access_policy is set to ExplicitRPCAccessPolicy then
+    endpoint methods need to be explicitly exposed.::
+
+        # foo() cannot be invoked by an RPC client
+        def foo(self):
+            pass
+
+        # bar() can be invoked by an RPC client
+        @rpc.expose
+        def bar(self):
+            pass
+
+    """
+
+    func.exposed = True
+
+    return func
