@@ -30,6 +30,7 @@ import logging
 import threading
 import traceback
 
+import debtcollector
 from oslo_config import cfg
 from oslo_service import service
 from oslo_utils import eventletutils
@@ -38,7 +39,7 @@ import six
 from stevedore import driver
 
 from oslo_messaging._drivers import base as driver_base
-from oslo_messaging._i18n import _LW, _LI
+from oslo_messaging._i18n import _LW
 from oslo_messaging import exceptions
 
 LOG = logging.getLogger(__name__)
@@ -52,7 +53,8 @@ _pool_opts = [
     cfg.IntOpt('executor_thread_pool_size',
                default=64,
                deprecated_name="rpc_thread_pool_size",
-               help='Size of executor thread pool.'),
+               help='Size of executor thread pool when'
+               ' executor is threading or eventlet.'),
 ]
 
 
@@ -321,8 +323,8 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         :param dispatcher: has a dispatch() method which is invoked for each
                            incoming request
         :type dispatcher: DispatcherBase
-        :param executor: name of message executor - for example
-                         'eventlet', 'blocking'
+        :param executor: name of message executor - available values are
+                         'eventlet' and 'threading'
         :type executor: str
         """
         self.conf = transport.conf
@@ -332,13 +334,15 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         self.dispatcher = dispatcher
         self.executor_type = executor
         if self.executor_type == 'blocking':
-            # NOTE(sileht): We keep blocking as default to not enforce the
-            # application to use threading or eventlet. Because application
-            # have to be preprepared accordingly for each one (monkeypatching,
-            # threadsafe, ...)
-            LOG.info(_LI("blocking executor handles only one message at "
-                         "once. threading or eventlet executor is "
-                         "recommended."))
+            debtcollector.deprecate(
+                'blocking executor is deprecated. Executor default will be '
+                'removed. Use explicitly threading or eventlet instead',
+                version="pike", removal_version="rocky",
+                category=FutureWarning)
+        elif self.executor_type == "eventlet":
+            eventletutils.warn_eventlet_not_patched(
+                expected_patched_modules=['thread'],
+                what="the 'oslo.messaging eventlet executor'")
 
         self.listener = None
 
@@ -391,29 +395,19 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         choose to dispatch messages in a new thread, coroutine or simply the
         current thread.
         """
-        # Warn that restarting will be deprecated
         if self._started:
-            LOG.warning(_LW('Restarting a MessageHandlingServer is inherently '
-                            'racy. It is deprecated, and will become a noop '
-                            'in a future release of oslo.messaging. If you '
-                            'need to restart MessageHandlingServer you should '
-                            'instantiate a new object.'))
+            LOG.warning(_LW('The server has already been started. Ignoring'
+                            ' the redundant call to start().'))
+            return
+
         self._started = True
 
         executor_opts = {}
 
-        if self.executor_type == "threading":
+        if self.executor_type in ("threading", "eventlet"):
             executor_opts["max_workers"] = (
                 override_pool_size or self.conf.executor_thread_pool_size
             )
-        elif self.executor_type == "eventlet":
-            eventletutils.warn_eventlet_not_patched(
-                expected_patched_modules=['thread'],
-                what="the 'oslo.messaging eventlet executor'")
-            executor_opts["max_workers"] = (
-                override_pool_size or self.conf.executor_thread_pool_size
-            )
-
         self._work_executor = self._executor_cls(**executor_opts)
 
         try:
