@@ -22,41 +22,20 @@ import uuid
 import fixtures
 import kombu
 import kombu.transport.memory
-from oslo_config import cfg
 from oslo_serialization import jsonutils
-from oslo_utils import versionutils
-from oslotest import mockpatch
-import pkg_resources
 import testscenarios
 
 import oslo_messaging
 from oslo_messaging._drivers import amqpdriver
 from oslo_messaging._drivers import common as driver_common
 from oslo_messaging._drivers import impl_rabbit as rabbit_driver
+from oslo_messaging.exceptions import MessageDeliveryFailure
 from oslo_messaging.tests import utils as test_utils
 from six.moves import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class TestDeprecatedRabbitDriverLoad(test_utils.BaseTestCase):
-
-    def setUp(self):
-        super(TestDeprecatedRabbitDriverLoad, self).setUp(
-            conf=cfg.ConfigOpts())
-        self.messaging_conf.transport_driver = 'rabbit'
-        self.config(fake_rabbit=True, group="oslo_messaging_rabbit")
-
-    def test_driver_load(self):
-        transport = oslo_messaging.get_transport(self.conf)
-        self.addCleanup(transport.cleanup)
-        driver = transport._driver
-        url = driver._get_connection()._url
-
-        self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
-        self.assertEqual('memory:////', url)
-
-
 class TestHeartbeat(test_utils.BaseTestCase):
 
     @mock.patch('oslo_messaging._drivers.impl_rabbit.LOG')
@@ -106,7 +85,7 @@ class TestHeartbeat(test_utils.BaseTestCase):
 
     def test_test_heartbeat_sent_connection_fail(self):
         self._do_test_heartbeat_sent(
-            heartbeat_side_effect=kombu.exceptions.ConnectionError,
+            heartbeat_side_effect=kombu.exceptions.OperationalError,
             info='A recoverable connection/channel error occurred, '
             'trying to reconnect: %s')
 
@@ -139,12 +118,9 @@ class TestRabbitQos(test_utils.BaseTestCase):
 class TestRabbitDriverLoad(test_utils.BaseTestCase):
 
     scenarios = [
-        ('rabbit', dict(transport_driver='rabbit',
-                        url='amqp://guest:guest@localhost:5672//')),
-        ('kombu', dict(transport_driver='kombu',
-                       url='amqp://guest:guest@localhost:5672//')),
-        ('rabbit+memory', dict(transport_driver='kombu+memory',
-                               url='memory:///'))
+        ('rabbit', dict(transport_url='rabbit:/guest:guest@localhost:5672//')),
+        ('kombu', dict(transport_url='kombu:/guest:guest@localhost:5672//')),
+        ('rabbit+memory', dict(transport_url='kombu+memory:/'))
     ]
 
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
@@ -152,28 +128,26 @@ class TestRabbitDriverLoad(test_utils.BaseTestCase):
     def test_driver_load(self, fake_ensure, fake_reset):
         self.config(heartbeat_timeout_threshold=60,
                     group='oslo_messaging_rabbit')
-        self.messaging_conf.transport_driver = self.transport_driver
+        self.messaging_conf.transport_url = self.transport_url
         transport = oslo_messaging.get_transport(self.conf)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
-        url = driver._get_connection()._url
 
         self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
-        self.assertEqual(self.url, url)
 
 
 class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
     scenarios = [
         ('no_ssl', dict(options=dict(), expected=False)),
-        ('no_ssl_with_options', dict(options=dict(kombu_ssl_version='TLSv1'),
+        ('no_ssl_with_options', dict(options=dict(ssl_version='TLSv1'),
                                      expected=False)),
-        ('just_ssl', dict(options=dict(rabbit_use_ssl=True),
+        ('just_ssl', dict(options=dict(ssl=True),
                           expected=True)),
-        ('ssl_with_options', dict(options=dict(rabbit_use_ssl=True,
-                                               kombu_ssl_version='TLSv1',
-                                               kombu_ssl_keyfile='foo',
-                                               kombu_ssl_certfile='bar',
-                                               kombu_ssl_ca_certs='foobar'),
+        ('ssl_with_options', dict(options=dict(ssl=True,
+                                               ssl_version='TLSv1',
+                                               ssl_key_file='foo',
+                                               ssl_cert_file='bar',
+                                               ssl_ca_file='foobar'),
                                   expected=dict(ssl_version=3,
                                                 keyfile='foo',
                                                 certfile='bar',
@@ -189,16 +163,16 @@ class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
                                                  'kombu+memory:////')
         self.addCleanup(transport.cleanup)
 
-        transport._driver._get_connection()
+        connection = transport._driver._get_connection()
         connection_klass.assert_called_once_with(
             'memory:///', transport_options={
                 'client_properties': {
                     'capabilities': {
                         'connection.blocked': True,
                         'consumer_cancel_notify': True,
-                        'authentication_failure_close': True
-                    }
-                },
+                        'authentication_failure_close': True,
+                    },
+                    'connection_name': connection.name},
                 'confirm_publish': True,
                 'on_blocked': mock.ANY,
                 'on_unblocked': mock.ANY},
@@ -219,23 +193,11 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
             conn._publish(exchange_mock, 'msg', routing_key='routing_key',
                           timeout=1)
 
-        # NOTE(gcb) kombu accept TTL as seconds instead of millisecond since
-        # version 3.0.25, so do conversion according to kombu version.
-        # TODO(gcb) remove this workaround when all supported branches
-        # with requirement kombu >=3.0.25
-        kombu_version = pkg_resources.get_distribution('kombu').version
-        if versionutils.is_compatible('3.0.25', kombu_version):
-            fake_publish.assert_called_with(
-                'msg', expiration=1,
-                exchange=exchange_mock,
-                compression=self.conf.oslo_messaging_rabbit.kombu_compression,
-                routing_key='routing_key')
-        else:
-            fake_publish.assert_called_with(
-                'msg', expiration=1000,
-                exchange=exchange_mock,
-                compression=self.conf.oslo_messaging_rabbit.kombu_compression,
-                routing_key='routing_key')
+        fake_publish.assert_called_with(
+            'msg', expiration=1,
+            exchange=exchange_mock,
+            compression=self.conf.oslo_messaging_rabbit.kombu_compression,
+            routing_key='routing_key')
 
     @mock.patch('kombu.messaging.Producer.publish')
     def test_send_no_timeout(self, fake_publish):
@@ -279,7 +241,8 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
 
             with mock.patch('kombu.transport.virtual.Channel.close'):
                 # Ensure the exchange does not exists
-                self.assertRaises(exc, try_send, e_passive)
+                self.assertRaises(oslo_messaging.MessageDeliveryFailure,
+                                  try_send, e_passive)
                 # Create it
                 try_send(e_active)
                 # Ensure it creates it
@@ -287,16 +250,32 @@ class TestRabbitPublisher(test_utils.BaseTestCase):
 
             with mock.patch('kombu.messaging.Producer.publish',
                             side_effect=exc):
-                # Ensure the exchange is already in cache
-                self.assertIn('foobar', conn._declared_exchanges)
-                # Reset connection
-                self.assertRaises(exc, try_send, e_passive)
-                # Ensure the cache is empty
-                self.assertEqual(0, len(conn._declared_exchanges))
+                with mock.patch('kombu.transport.virtual.Channel.close'):
+                    # Ensure the exchange is already in cache
+                    self.assertIn('foobar', conn._declared_exchanges)
+                    # Reset connection
+                    self.assertRaises(oslo_messaging.MessageDeliveryFailure,
+                                      try_send, e_passive)
+                    # Ensure the cache is empty
+                    self.assertEqual(0, len(conn._declared_exchanges))
 
             try_send(e_active)
             self.assertIn('foobar', conn._declared_exchanges)
 
+    def test_send_exception_remap(self):
+        bad_exc = Exception("Non-oslo.messaging exception")
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        exchange_mock = mock.Mock()
+        with transport._driver._get_connection(
+                driver_common.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            with mock.patch('kombu.messaging.Producer.publish',
+                            side_effect=bad_exc):
+                self.assertRaises(MessageDeliveryFailure,
+                                  conn._ensure_publishing,
+                                  conn._publish, exchange_mock, 'msg')
+
 
 class TestRabbitConsume(test_utils.BaseTestCase):
 
@@ -336,7 +315,7 @@ class TestRabbitConsume(test_utils.BaseTestCase):
                 conn.connection.connection.recoverable_connection_errors = ()
                 conn.connection.connection.recoverable_channel_errors = ()
                 self.assertEqual(1, declare.call_count)
-                conn.connection.connection.transport.drain_events = mock.Mock()
+                conn.connection.connection.drain_events = mock.Mock()
                 # Ensure that a queue will be re-declared if the consume method
                 # of kombu.Queue raise amqp.NotFound
                 conn.consume()
@@ -360,7 +339,7 @@ class TestRabbitConsume(test_utils.BaseTestCase):
                     IOError,)
                 conn.connection.connection.recoverable_channel_errors = ()
                 self.assertEqual(1, declare.call_count)
-                conn.connection.connection.transport.drain_events = mock.Mock()
+                conn.connection.connection.drain_events = mock.Mock()
                 # Ensure that a queue will be re-declared after
                 # 'queue not found' exception despite on connection error.
                 conn.consume()
@@ -386,7 +365,7 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
 
     scenarios = [
         ('none', dict(url=None,
-                      expected=["amqp://guest:guest@localhost:5672//"])),
+                      expected=["amqp://guest:guest@localhost:5672/"])),
         ('memory', dict(url='kombu+memory:////',
                         expected=["memory:///"])),
         ('empty',
@@ -423,7 +402,7 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestRabbitTransportURL, self).setUp()
-        self.messaging_conf.transport_driver = 'rabbit'
+        self.messaging_conf.transport_url = 'rabbit:/'
         self.config(heartbeat_timeout_threshold=0,
                     group='oslo_messaging_rabbit')
 
@@ -466,8 +445,11 @@ class TestSendReceive(test_utils.BaseTestCase):
     ]
 
     _timeout = [
-        ('no_timeout', dict(timeout=None)),
-        ('timeout', dict(timeout=0.01)),  # FIXME(markmc): timeout=0 is broken?
+        ('no_timeout', dict(timeout=None, call_monitor_timeout=None)),
+        ('timeout', dict(timeout=0.01,  # FIXME(markmc): timeout=0 is broken?
+                         call_monitor_timeout=None)),
+        ('call_monitor_timeout', dict(timeout=0.01,
+                                      call_monitor_timeout=0.02)),
     ]
 
     @classmethod
@@ -497,15 +479,20 @@ class TestSendReceive(test_utils.BaseTestCase):
         replies = []
         msgs = []
 
+        # FIXME(danms): Surely this is not the right way to do this...
+        self.ctxt['client_timeout'] = self.call_monitor_timeout
+
         def send_and_wait_for_reply(i):
             try:
 
                 timeout = self.timeout
+                cm_timeout = self.call_monitor_timeout
                 replies.append(driver.send(target,
                                            self.ctxt,
                                            {'tx_id': i},
                                            wait_for_reply=True,
-                                           timeout=timeout))
+                                           timeout=timeout,
+                                           call_monitor_timeout=cm_timeout))
                 self.assertFalse(self.failure)
                 self.assertIsNone(self.timeout)
             except (ZeroDivisionError, oslo_messaging.MessagingTimeout) as e:
@@ -591,16 +578,18 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
         wait_conditions = []
         orig_reply_waiter = amqpdriver.ReplyWaiter.wait
 
-        def reply_waiter(self, msg_id, timeout):
+        def reply_waiter(self, msg_id, timeout, call_monitor_timeout):
             if wait_conditions:
                 cond = wait_conditions.pop()
                 with cond:
                     cond.notify()
                 with cond:
                     cond.wait()
-            return orig_reply_waiter(self, msg_id, timeout)
+            return orig_reply_waiter(self, msg_id, timeout,
+                                     call_monitor_timeout)
 
-        self.stubs.Set(amqpdriver.ReplyWaiter, 'wait', reply_waiter)
+        self.useFixture(fixtures.MockPatchObject(
+            amqpdriver.ReplyWaiter, 'wait', reply_waiter))
 
         def send_and_wait_for_reply(i, wait_for_reply):
             replies.append(driver.send(target,
@@ -654,7 +643,7 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
         # Verify the _send_reply was not invoked by driver:
         with mock.patch.object(msgs[2], '_send_reply') as method:
             msgs[2].reply({'rx_id': 2})
-            self.assertEqual(method.call_count, 0)
+            self.assertEqual(0, method.call_count)
 
         # Wait for the 3rd thread to finish
         senders[2].join()
@@ -887,11 +876,12 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
     ]
 
     _context = [
-        ('empty_ctxt', dict(ctxt={}, expected_ctxt={})),
+        ('empty_ctxt', dict(ctxt={}, expected_ctxt={'client_timeout': None})),
         ('user_project_ctxt',
          dict(ctxt={'_context_user': 'mark',
                     '_context_project': 'snarkybunch'},
-              expected_ctxt={'user': 'mark', 'project': 'snarkybunch'})),
+              expected_ctxt={'user': 'mark', 'project': 'snarkybunch',
+                             'client_timeout': None})),
     ]
 
     _compression = [
@@ -936,6 +926,7 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
             '_msg_id': uuid.uuid4().hex,
             '_unique_id': uuid.uuid4().hex,
             '_reply_q': 'reply_' + uuid.uuid4().hex,
+            '_timeout': None,
         })
 
         msg['oslo.message'] = jsonutils.dumps(msg['oslo.message'])
@@ -963,19 +954,17 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
                     heartbeat_timeout_threshold=0,
                     group="oslo_messaging_rabbit")
 
-        self.kombu_connect = mock.Mock()
-        self.useFixture(mockpatch.Patch(
-            'kombu.connection.Connection.connect',
-            side_effect=self.kombu_connect))
-        self.useFixture(mockpatch.Patch(
+        self.useFixture(fixtures.MockPatch(
             'kombu.connection.Connection.connection'))
-        self.useFixture(mockpatch.Patch(
+        self.useFixture(fixtures.MockPatch(
             'kombu.connection.Connection.channel'))
 
         # starting from the first broker in the list
         url = oslo_messaging.TransportURL.parse(self.conf, None)
         self.connection = rabbit_driver.Connection(self.conf, url,
                                                    driver_common.PURPOSE_SEND)
+        self.useFixture(fixtures.MockPatch(
+            'kombu.connection.Connection.connect'))
         self.addCleanup(self.connection.close)
 
     def test_ensure_four_retry(self):
@@ -983,7 +972,6 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
         self.assertRaises(oslo_messaging.MessageDeliveryFailure,
                           self.connection.ensure, mock_callback,
                           retry=4)
-        self.assertEqual(5, self.kombu_connect.call_count)
         self.assertEqual(6, mock_callback.call_count)
 
     def test_ensure_one_retry(self):
@@ -991,7 +979,6 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
         self.assertRaises(oslo_messaging.MessageDeliveryFailure,
                           self.connection.ensure, mock_callback,
                           retry=1)
-        self.assertEqual(2, self.kombu_connect.call_count)
         self.assertEqual(3, mock_callback.call_count)
 
     def test_ensure_no_retry(self):
@@ -999,7 +986,6 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
         self.assertRaises(oslo_messaging.MessageDeliveryFailure,
                           self.connection.ensure, mock_callback,
                           retry=0)
-        self.assertEqual(1, self.kombu_connect.call_count)
         self.assertEqual(2, mock_callback.call_count)
 
 
