@@ -20,6 +20,7 @@ from oslo_config import cfg
 from six import moves
 
 import oslo_messaging
+from oslo_messaging._drivers.kafka_driver import kafka_options
 from oslo_messaging._drivers.zmq_driver import zmq_options
 from oslo_messaging.notify import notifier
 from oslo_messaging.tests import utils as test_utils
@@ -73,6 +74,24 @@ class TransportFixture(fixtures.Fixture):
         time.sleep(0.5)
 
 
+class RPCTransportFixture(TransportFixture):
+    """Fixture defined to setup RPC transport."""
+
+    def setUp(self):
+        super(RPCTransportFixture, self).setUp()
+        self.transport = oslo_messaging.get_rpc_transport(self.conf,
+                                                          url=self.url)
+
+
+class NotificationTransportFixture(TransportFixture):
+    """Fixture defined to setup notification transport."""
+
+    def setUp(self):
+        super(NotificationTransportFixture, self).setUp()
+        self.transport = oslo_messaging.get_notification_transport(
+            self.conf, url=self.url)
+
+
 class RpcServerFixture(fixtures.Fixture):
     """Fixture to setup the TestServerEndpoint."""
 
@@ -90,7 +109,7 @@ class RpcServerFixture(fixtures.Fixture):
     def setUp(self):
         super(RpcServerFixture, self).setUp()
         endpoints = [self.endpoint, self]
-        transport = self.useFixture(TransportFixture(self.conf, self.url))
+        transport = self.useFixture(RPCTransportFixture(self.conf, self.url))
         self.server = oslo_messaging.get_rpc_server(
             transport=transport.transport,
             target=self.target,
@@ -111,8 +130,9 @@ class RpcServerFixture(fixtures.Fixture):
 
     def _stop(self):
         self.thread.stop()
-        self._ctrl.cast({}, 'ping')
-        self.thread.join()
+        self.thread.join(timeout=30)
+        if self.thread.isAlive():
+            raise Exception("Server did not shutdown correctly")
 
     def ping(self, ctxt):
         pass
@@ -126,7 +146,7 @@ class RpcServerGroupFixture(fixtures.Fixture):
                  use_fanout_ctrl=False, endpoint=None):
         self.conf = conf
         self.url = url
-        # NOTE(sileht): topic and servier_name must be uniq
+        # NOTE(sileht): topic and server_name must be unique
         # to be able to run all tests in parallel
         self.topic = topic or str(uuid.uuid4())
         self.names = names or ["server_%i_%s" % (i, str(uuid.uuid4())[:8])
@@ -161,12 +181,12 @@ class RpcServerGroupFixture(fixtures.Fixture):
         else:
             if server == 'all':
                 target = self._target(fanout=True)
-            elif server >= 0 and server < len(self.targets):
+            elif 0 <= server < len(self.targets):
                 target = self.targets[server]
             else:
                 raise ValueError("Invalid value for server: %r" % server)
 
-        transport = self.useFixture(TransportFixture(self.conf, self.url))
+        transport = self.useFixture(RPCTransportFixture(self.conf, self.url))
         client = ClientStub(transport.transport, target, cast=cast,
                             timeout=5)
         transport.wait()
@@ -180,7 +200,7 @@ class RpcServerGroupFixture(fixtures.Fixture):
             if server == 'all':
                 for s in self.servers:
                     s.syncq.get(timeout=5)
-            elif server >= 0 and server < len(self.targets):
+            elif 0 <= server < len(self.targets):
                 self.servers[server].syncq.get(timeout=5)
             else:
                 raise ValueError("Invalid value for server: %r" % server)
@@ -284,15 +304,15 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
         driver = os.environ.get("TRANSPORT_DRIVER")
         if driver:
             self.url = os.environ.get('PIFPAF_URL')
-            if driver == "pika" and self.url:
-                self.url = self.url.replace("rabbit://", "pika://")
         else:
             self.url = os.environ.get('TRANSPORT_URL')
 
         if not self.url:
             self.skipTest("No transport url configured")
 
-        zmq_options.register_opts(conf)
+        transport_url = oslo_messaging.TransportURL.parse(conf, self.url)
+
+        zmq_options.register_opts(conf, transport_url)
 
         zmq_matchmaker = os.environ.get('ZMQ_MATCHMAKER')
         if zmq_matchmaker:
@@ -304,18 +324,26 @@ class SkipIfNoTransportURL(test_utils.BaseTestCase):
                         rpc_zmq_ipc_dir=zmq_ipc_dir)
         zmq_redis_port = os.environ.get('ZMQ_REDIS_PORT')
         if zmq_redis_port:
-            self.config(port=zmq_redis_port, group="matchmaker_redis")
-            self.config(check_timeout=10000, group="matchmaker_redis")
-            self.config(wait_timeout=1000, group="matchmaker_redis")
+            self.config(port=zmq_redis_port,
+                        check_timeout=10000,
+                        wait_timeout=1000,
+                        group="matchmaker_redis")
         zmq_use_pub_sub = os.environ.get('ZMQ_USE_PUB_SUB')
-        self.config(use_pub_sub=zmq_use_pub_sub,
-                    group='oslo_messaging_zmq')
         zmq_use_router_proxy = os.environ.get('ZMQ_USE_ROUTER_PROXY')
-        self.config(use_router_proxy=zmq_use_router_proxy,
-                    group='oslo_messaging_zmq')
         zmq_use_acks = os.environ.get('ZMQ_USE_ACKS')
-        self.config(rpc_use_acks=zmq_use_acks,
+        self.config(use_pub_sub=zmq_use_pub_sub,
+                    use_router_proxy=zmq_use_router_proxy,
+                    rpc_use_acks=zmq_use_acks,
                     group='oslo_messaging_zmq')
+        zmq_use_dynamic_connections = \
+            os.environ.get('ZMQ_USE_DYNAMIC_CONNECTIONS')
+        self.config(use_dynamic_connections=zmq_use_dynamic_connections,
+                    group='oslo_messaging_zmq')
+
+        kafka_options.register_opts(conf, transport_url)
+
+        self.config(producer_batch_size=0,
+                    group='oslo_messaging_kafka')
 
 
 class NotificationFixture(fixtures.Fixture):
@@ -333,9 +361,10 @@ class NotificationFixture(fixtures.Fixture):
         targets = [oslo_messaging.Target(topic=t) for t in self.topics]
         # add a special topic for internal notifications
         targets.append(oslo_messaging.Target(topic=self.name))
-        transport = self.useFixture(TransportFixture(self.conf, self.url))
+        transport = self.useFixture(NotificationTransportFixture(self.conf,
+                                                                 self.url))
         self.server = self._get_server(transport, targets)
-        self._ctrl = self.notifier('internal', topic=self.name)
+        self._ctrl = self.notifier('internal', topics=[self.name])
         self._start()
         transport.wait()
 
@@ -355,15 +384,17 @@ class NotificationFixture(fixtures.Fixture):
 
     def _stop(self):
         self.thread.stop()
-        self._ctrl.sample({}, 'shutdown', 'shutdown')
-        self.thread.join()
+        self.thread.join(timeout=30)
+        if self.thread.isAlive():
+            raise Exception("Server did not shutdown properly")
 
-    def notifier(self, publisher, topic=None):
-        transport = self.useFixture(TransportFixture(self.conf, self.url))
+    def notifier(self, publisher, topics=None):
+        transport = self.useFixture(NotificationTransportFixture(self.conf,
+                                                                 self.url))
         n = notifier.Notifier(transport.transport,
                               publisher,
                               driver='messaging',
-                              topic=topic or self.topics[0])
+                              topics=topics or self.topics)
         transport.wait()
         return n
 
