@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import fixtures
 import threading
 
 from oslo_config import cfg
@@ -22,7 +23,6 @@ import oslo_messaging
 from oslo_messaging.notify import dispatcher
 from oslo_messaging.notify import notifier as msg_notifier
 from oslo_messaging.tests import utils as test_utils
-import six
 from six.moves import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
@@ -115,9 +115,9 @@ class ListenerSetupMixin(object):
     def wait_for_messages(self, expect_messages, tracker_name='__default__'):
         self.trackers[tracker_name].wait_for_messages(expect_messages)
 
-    def _setup_notifier(self, transport, topic='testtopic',
+    def _setup_notifier(self, transport, topics=['testtopic'],
                         publisher_id='testpublisher'):
-        return oslo_messaging.Notifier(transport, topic=topic,
+        return oslo_messaging.Notifier(transport, topics=topics,
                                        driver='messaging',
                                        publisher_id=publisher_id)
 
@@ -131,8 +131,12 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
     def setUp(self):
         super(TestNotifyListener, self).setUp(conf=cfg.ConfigOpts())
         ListenerSetupMixin.setUp(self)
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
-    def test_constructor(self):
+    @mock.patch('debtcollector.deprecate')
+    def test_constructor(self, deprecate):
         transport = msg_notifier.get_notification_transport(
             self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo')
@@ -147,6 +151,11 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                               dispatcher.NotificationDispatcher)
         self.assertIs(listener.dispatcher.endpoints, endpoints)
         self.assertEqual('blocking', listener.executor_type)
+        deprecate.assert_called_once_with(
+            'blocking executor is deprecated. Executor default will be '
+            'removed. Use explicitly threading or eventlet instead',
+            removal_version='rocky', version='pike',
+            category=FutureWarning)
 
     def test_no_target_topic(self):
         transport = msg_notifier.get_notification_transport(
@@ -177,7 +186,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
             self.assertTrue(False)
 
     def test_batch_timeout(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.return_value = None
@@ -185,7 +195,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, 1))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(12):
+        for _ in range(12):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(3)
@@ -203,7 +213,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                         mock.call(messages * 2)])
 
     def test_batch_size(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.return_value = None
@@ -211,7 +222,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, None))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(10):
+        for _ in range(10):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(2)
@@ -228,7 +239,8 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                         mock.call(messages * 5)])
 
     def test_batch_size_exception_path(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_notification_transport(self.conf,
+                                                              url='fake:')
 
         endpoint = mock.Mock()
         endpoint.info.side_effect = [None, Exception('boom!')]
@@ -236,7 +248,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                                                batch=(5, None))
 
         notifier = self._setup_notifier(transport)
-        for i in six.moves.range(10):
+        for _ in range(10):
             notifier.info({}, 'an_event.start', 'test message')
 
         self.wait_for_messages(2)
@@ -279,9 +291,9 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                    oslo_messaging.Target(topic="topic2")]
         listener_thread = self._setup_listener(transport, [endpoint],
                                                targets=targets)
-        notifier = self._setup_notifier(transport, topic='topic1')
+        notifier = self._setup_notifier(transport, topics=['topic1'])
         notifier.info({'ctxt': '1'}, 'an_event.start1', 'test')
-        notifier = self._setup_notifier(transport, topic='topic2')
+        notifier = self._setup_notifier(transport, topics=['topic2'])
         notifier.info({'ctxt': '2'}, 'an_event.start2', 'test')
 
         self.wait_for_messages(2)
@@ -309,7 +321,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
         listener_thread = self._setup_listener(transport, [endpoint],
                                                targets=targets)
 
-        notifier = self._setup_notifier(transport, topic="topic")
+        notifier = self._setup_notifier(transport, topics=["topic"])
 
         def mock_notifier_exchange(name):
             def side_effect(target, ctxt, message, version, retry):
@@ -407,7 +419,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
         listener2_thread = self._setup_listener(transport, [endpoint2],
                                                 targets=targets, pool="pool2")
 
-        notifier = self._setup_notifier(transport, topic="topic")
+        notifier = self._setup_notifier(transport, topics=["topic"])
         notifier.info({'ctxt': '0'}, 'an_event.start', 'test message0')
         notifier.info({'ctxt': '1'}, 'an_event.start', 'test message1')
 
@@ -450,7 +462,7 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
                              'an_event.start', 'test message%d' % i,
                              {'timestamp': mock.ANY, 'message_id': mock.ANY})
 
-        notifier = self._setup_notifier(transport, topic="topic")
+        notifier = self._setup_notifier(transport, topics=["topic"])
         mocked_endpoint1_calls = []
         for i in range(0, 25):
             notifier.info({'ctxt': '%d' % i}, 'an_event.start',
@@ -500,3 +512,18 @@ class TestNotifyListener(test_utils.BaseTestCase, ListenerSetupMixin):
         for call in mocked_endpoint1_calls:
             self.assertIn(call, endpoint2.info.mock_calls +
                           endpoint3.info.mock_calls)
+
+
+class TestListenerTransportWarning(test_utils.BaseTestCase):
+
+    @mock.patch('oslo_messaging.notify.listener.LOG')
+    def test_warning_when_rpc_transport(self, log):
+        transport = oslo_messaging.get_rpc_transport(self.conf)
+        target = oslo_messaging.Target(topic='foo')
+        endpoints = [object()]
+        oslo_messaging.get_notification_listener(
+            transport, [target], endpoints)
+        log.warning.assert_called_once_with(
+            "Using RPC transport for notifications. Please use "
+            "get_notification_transport to obtain a "
+            "notification transport instance.")
