@@ -14,6 +14,7 @@
 #    under the License.
 
 from oslo_config import cfg
+from six.moves import mock
 import testscenarios
 
 import oslo_messaging
@@ -24,15 +25,6 @@ from oslo_messaging.tests import utils as test_utils
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class _FakeTransport(object):
-
-    def __init__(self, conf):
-        self.conf = conf
-
-    def _send(self, *args, **kwargs):
-        pass
-
-
 class TestCastCall(test_utils.BaseTestCase):
 
     scenarios = [
@@ -51,23 +43,26 @@ class TestCastCall(test_utils.BaseTestCase):
     def test_cast_call(self):
         self.config(rpc_response_timeout=None)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, oslo_messaging.Target())
 
-        self.mox.StubOutWithMock(transport, '_send')
+        transport._send = mock.Mock()
 
         msg = dict(method='foo', args=self.args)
         kwargs = {'retry': None}
         if self.call:
             kwargs['wait_for_reply'] = True
             kwargs['timeout'] = None
-
-        transport._send(oslo_messaging.Target(), self.ctxt, msg, **kwargs)
-        self.mox.ReplayAll()
+            kwargs['call_monitor_timeout'] = None
 
         method = client.call if self.call else client.cast
         method(self.ctxt, 'foo', **self.args)
 
+        transport._send.assert_called_once_with(oslo_messaging.Target(),
+                                                self.ctxt,
+                                                msg,
+                                                **kwargs)
+
 
 class TestCastToTarget(test_utils.BaseTestCase):
 
@@ -188,19 +183,16 @@ class TestCastToTarget(test_utils.BaseTestCase):
         target = oslo_messaging.Target(**self.ctor)
         expect_target = oslo_messaging.Target(**self.expect)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, target)
 
-        self.mox.StubOutWithMock(transport, '_send')
+        transport._send = mock.Mock()
 
         msg = dict(method='foo', args={})
         if 'namespace' in self.expect:
             msg['namespace'] = self.expect['namespace']
         if 'version' in self.expect:
             msg['version'] = self.expect['version']
-        transport._send(expect_target, {}, msg, retry=None)
-
-        self.mox.ReplayAll()
 
         if self.prepare:
             client = client.prepare(**self.prepare)
@@ -208,6 +200,11 @@ class TestCastToTarget(test_utils.BaseTestCase):
                 client = client.prepare(**self.prepare)
         client.cast({}, 'foo')
 
+        transport._send.assert_called_once_with(expect_target,
+                                                {},
+                                                msg,
+                                                retry=None)
+
 
 TestCastToTarget.generate_scenarios()
 
@@ -219,40 +216,46 @@ class TestCallTimeout(test_utils.BaseTestCase):
 
     scenarios = [
         ('all_none',
-         dict(confval=None, ctor=None, prepare=_notset, expect=None)),
+         dict(confval=None, ctor=None, prepare=_notset, expect=None, cm=None)),
         ('confval',
-         dict(confval=21, ctor=None, prepare=_notset, expect=21)),
+         dict(confval=21, ctor=None, prepare=_notset, expect=21, cm=None)),
         ('ctor',
-         dict(confval=None, ctor=21.1, prepare=_notset, expect=21.1)),
+         dict(confval=None, ctor=21.1, prepare=_notset, expect=21.1, cm=None)),
         ('ctor_zero',
-         dict(confval=None, ctor=0, prepare=_notset, expect=0)),
+         dict(confval=None, ctor=0, prepare=_notset, expect=0, cm=None)),
         ('prepare',
-         dict(confval=None, ctor=None, prepare=21.1, expect=21.1)),
+         dict(confval=None, ctor=None, prepare=21.1, expect=21.1, cm=None)),
         ('prepare_override',
-         dict(confval=None, ctor=10.1, prepare=21.1, expect=21.1)),
+         dict(confval=None, ctor=10.1, prepare=21.1, expect=21.1, cm=None)),
         ('prepare_zero',
-         dict(confval=None, ctor=None, prepare=0, expect=0)),
+         dict(confval=None, ctor=None, prepare=0, expect=0, cm=None)),
+        ('call_monitor',
+         dict(confval=None, ctor=None, prepare=60, expect=60, cm=30)),
     ]
 
     def test_call_timeout(self):
         self.config(rpc_response_timeout=self.confval)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
-                                          timeout=self.ctor)
+                                          timeout=self.ctor,
+                                          call_monitor_timeout=self.cm)
 
-        self.mox.StubOutWithMock(transport, '_send')
+        transport._send = mock.Mock()
 
         msg = dict(method='foo', args={})
-        kwargs = dict(wait_for_reply=True, timeout=self.expect, retry=None)
-        transport._send(oslo_messaging.Target(), {}, msg, **kwargs)
-
-        self.mox.ReplayAll()
+        kwargs = dict(wait_for_reply=True, timeout=self.expect, retry=None,
+                      call_monitor_timeout=self.cm)
 
         if self.prepare is not _notset:
             client = client.prepare(timeout=self.prepare)
         client.call({}, 'foo')
 
+        transport._send.assert_called_once_with(oslo_messaging.Target(),
+                                                {},
+                                                msg,
+                                                **kwargs)
+
 
 class TestCallRetry(test_utils.BaseTestCase):
 
@@ -266,23 +269,25 @@ class TestCallRetry(test_utils.BaseTestCase):
     ]
 
     def test_call_retry(self):
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
                                           retry=self.ctor)
 
-        self.mox.StubOutWithMock(transport, '_send')
+        transport._send = mock.Mock()
 
         msg = dict(method='foo', args={})
         kwargs = dict(wait_for_reply=True, timeout=60,
-                      retry=self.expect)
-        transport._send(oslo_messaging.Target(), {}, msg, **kwargs)
-
-        self.mox.ReplayAll()
+                      retry=self.expect, call_monitor_timeout=None)
 
         if self.prepare is not _notset:
             client = client.prepare(retry=self.prepare)
         client.call({}, 'foo')
 
+        transport._send.assert_called_once_with(oslo_messaging.Target(),
+                                                {},
+                                                msg,
+                                                **kwargs)
+
 
 class TestCallFanout(test_utils.BaseTestCase):
 
@@ -293,7 +298,7 @@ class TestCallFanout(test_utils.BaseTestCase):
     ]
 
     def test_call_fanout(self):
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport,
                                           oslo_messaging.Target(**self.target))
 
@@ -322,43 +327,55 @@ class TestSerializer(test_utils.BaseTestCase):
     def test_call_serializer(self):
         self.config(rpc_response_timeout=None)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         serializer = msg_serializer.NoOpSerializer()
 
         client = oslo_messaging.RPCClient(transport, oslo_messaging.Target(),
                                           serializer=serializer)
 
-        self.mox.StubOutWithMock(transport, '_send')
+        transport._send = mock.Mock()
 
-        msg = dict(method='foo',
-                   args=dict([(k, 's' + v) for k, v in self.args.items()]))
         kwargs = dict(wait_for_reply=True, timeout=None) if self.call else {}
         kwargs['retry'] = None
-        transport._send(oslo_messaging.Target(),
-                        dict(user='alice'),
-                        msg,
-                        **kwargs).AndReturn(self.retval)
+        if self.call:
+            kwargs['call_monitor_timeout'] = None
 
-        self.mox.StubOutWithMock(serializer, 'serialize_entity')
-        self.mox.StubOutWithMock(serializer, 'deserialize_entity')
-        self.mox.StubOutWithMock(serializer, 'serialize_context')
+        transport._send.return_value = self.retval
 
-        for arg in self.args:
-            serializer.serialize_entity(self.ctxt, arg).AndReturn('s' + arg)
+        serializer.serialize_entity = mock.Mock()
+        serializer.deserialize_entity = mock.Mock()
+        serializer.serialize_context = mock.Mock()
+
+        def _stub(ctxt, arg):
+            return 's' + arg
+
+        msg = dict(method='foo', args=dict())
+        for k, v in self.args.items():
+            msg['args'][k] = 's' + v
+        serializer.serialize_entity.side_effect = _stub
 
         if self.call:
-            serializer.deserialize_entity(self.ctxt, self.retval).\
-                AndReturn('d' + self.retval)
+            serializer.deserialize_entity.return_value = 'd' + self.retval
 
-        serializer.serialize_context(self.ctxt).AndReturn(dict(user='alice'))
-
-        self.mox.ReplayAll()
+        serializer.serialize_context.return_value = dict(user='alice')
 
         method = client.call if self.call else client.cast
         retval = method(self.ctxt, 'foo', **self.args)
         if self.retval is not None:
             self.assertEqual('d' + self.retval, retval)
 
+        transport._send.assert_called_once_with(oslo_messaging.Target(),
+                                                dict(user='alice'),
+                                                msg,
+                                                **kwargs)
+        expected_calls = [mock.call(self.ctxt, arg) for arg in self.args]
+        self.assertEqual(expected_calls,
+                         serializer.serialize_entity.mock_calls)
+        if self.call:
+            serializer.deserialize_entity.assert_called_once_with(self.ctxt,
+                                                                  self.retval)
+        serializer.serialize_context.assert_called_once_with(self.ctxt)
+
 
 class TestVersionCap(test_utils.BaseTestCase):
 
@@ -411,14 +428,14 @@ class TestVersionCap(test_utils.BaseTestCase):
     def test_version_cap(self):
         self.config(rpc_response_timeout=None)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         target = oslo_messaging.Target(version=self.version)
         client = oslo_messaging.RPCClient(transport, target,
                                           version_cap=self.cap)
 
         if self.success:
-            self.mox.StubOutWithMock(transport, '_send')
+            transport._send = mock.Mock()
 
             if self.prepare_version is not _notset:
                 target = target(version=self.prepare_version)
@@ -431,10 +448,7 @@ class TestVersionCap(test_utils.BaseTestCase):
             if self.call:
                 kwargs['wait_for_reply'] = True
                 kwargs['timeout'] = None
-
-            transport._send(target, {}, msg, **kwargs)
-
-            self.mox.ReplayAll()
+                kwargs['call_monitor_timeout'] = None
 
         prep_kwargs = {}
         if self.prepare_cap is not _notset:
@@ -452,7 +466,7 @@ class TestVersionCap(test_utils.BaseTestCase):
             self.assertFalse(self.success)
         else:
             self.assertTrue(self.success)
-
+            transport._send.assert_called_once_with(target, {}, msg, **kwargs)
 
 TestVersionCap.generate_scenarios()
 
@@ -520,7 +534,7 @@ class TestCanSendVersion(test_utils.BaseTestCase):
     def test_version_cap(self):
         self.config(rpc_response_timeout=None)
 
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         target = oslo_messaging.Target(version=self.version)
         client = oslo_messaging.RPCClient(transport, target,
@@ -546,7 +560,7 @@ class TestCanSendVersion(test_utils.BaseTestCase):
 
     def test_invalid_version_type(self):
         target = oslo_messaging.Target(topic='sometopic')
-        transport = _FakeTransport(self.conf)
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         client = oslo_messaging.RPCClient(transport, target)
         self.assertRaises(exceptions.MessagingException,
                           client.prepare, version='5')
@@ -554,3 +568,15 @@ class TestCanSendVersion(test_utils.BaseTestCase):
                           client.prepare, version='5.a')
         self.assertRaises(exceptions.MessagingException,
                           client.prepare, version='5.5.a')
+
+
+class TestTransportWarning(test_utils.BaseTestCase):
+
+    @mock.patch('oslo_messaging.rpc.client.LOG')
+    def test_warning_when_notifier_transport(self, log):
+        transport = oslo_messaging.get_notification_transport(self.conf)
+        oslo_messaging.RPCClient(transport, oslo_messaging.Target())
+        log.warning.assert_called_once_with(
+            "Using notification transport for RPC. Please use "
+            "get_rpc_transport to obtain an RPC transport "
+            "instance.")
