@@ -13,14 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import eventlet
 import threading
+import warnings
 
+import eventlet
+import fixtures
 from oslo_config import cfg
+from six.moves import mock
 import testscenarios
 
-import mock
 import oslo_messaging
+from oslo_messaging import rpc
+from oslo_messaging.rpc import dispatcher
 from oslo_messaging.rpc import server as rpc_server_module
 from oslo_messaging import server as server_module
 from oslo_messaging.tests import utils as test_utils
@@ -31,9 +35,11 @@ load_tests = testscenarios.load_tests_apply_scenarios
 class ServerSetupMixin(object):
 
     class Server(object):
-        def __init__(self, transport, topic, server, endpoint, serializer):
+        def __init__(self, transport, topic, server, endpoint, serializer,
+                     exchange):
             self.controller = ServerSetupMixin.ServerController()
-            target = oslo_messaging.Target(topic=topic, server=server)
+            target = oslo_messaging.Target(topic=topic, server=server,
+                                           exchange=exchange)
             self.server = oslo_messaging.get_rpc_server(transport,
                                                         target,
                                                         [endpoint,
@@ -77,25 +83,25 @@ class ServerSetupMixin(object):
     def __init__(self):
         self.serializer = self.TestSerializer()
 
-    def _setup_server(self, transport, endpoint, topic=None, server=None):
+    def _setup_server(self, transport, endpoint, topic=None, server=None,
+                      exchange=None):
         server = self.Server(transport,
                              topic=topic or 'testtopic',
                              server=server or 'testserver',
                              endpoint=endpoint,
-                             serializer=self.serializer)
+                             serializer=self.serializer,
+                             exchange=exchange)
 
         server.start()
         return server
 
-    def _stop_server(self, client, server, topic=None):
-        if topic is not None:
-            client = client.prepare(topic=topic)
+    def _stop_server(self, client, server, topic=None, exchange=None):
         client.cast({}, 'stop')
         server.wait()
 
-    def _setup_client(self, transport, topic='testtopic'):
-        return oslo_messaging.RPCClient(transport,
-                                        oslo_messaging.Target(topic=topic),
+    def _setup_client(self, transport, topic='testtopic', exchange=None):
+        target = oslo_messaging.Target(topic=topic, exchange=exchange)
+        return oslo_messaging.RPCClient(transport, target=target,
                                         serializer=self.serializer)
 
 
@@ -107,25 +113,60 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
 
     def setUp(self):
         super(TestRPCServer, self).setUp(conf=cfg.ConfigOpts())
+        # FakeExchangeManager uses a class-level exchanges mapping; "reset" it
+        # before tests assert amount of items stored
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
-    def test_constructor(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+    @mock.patch('warnings.warn')
+    def test_constructor(self, warn):
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
+        access_policy = dispatcher.DefaultRPCAccessPolicy
 
-        server = oslo_messaging.get_rpc_server(transport, target, endpoints,
-                                               serializer=serializer)
-
+        warnings.simplefilter("always", FutureWarning)
+        server = oslo_messaging.get_rpc_server(transport,
+                                               target,
+                                               endpoints,
+                                               serializer=serializer,
+                                               access_policy=access_policy)
         self.assertIs(server.conf, self.conf)
         self.assertIs(server.transport, transport)
         self.assertIsInstance(server.dispatcher, oslo_messaging.RPCDispatcher)
         self.assertIs(server.dispatcher.endpoints, endpoints)
         self.assertIs(server.dispatcher.serializer, serializer)
         self.assertEqual('blocking', server.executor_type)
+        self.assertEqual([
+            mock.call("blocking executor is deprecated. Executor default will "
+                      "be removed. Use explicitly threading or eventlet "
+                      "instead in version 'pike' and will be removed in "
+                      "version 'rocky'",
+                      category=FutureWarning, stacklevel=3)
+        ], warn.mock_calls)
+
+    @mock.patch('warnings.warn')
+    def test_constructor_without_explicit_RPCAccessPolicy(self, warn):
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
+        target = oslo_messaging.Target(topic='foo', server='bar')
+        endpoints = [object()]
+        serializer = object()
+
+        warnings.simplefilter("always", FutureWarning)
+        oslo_messaging.get_rpc_server(transport, target,
+                                      endpoints, serializer=serializer)
+        self.assertEqual([
+            mock.call("blocking executor is deprecated. Executor default will "
+                      "be removed. Use explicitly threading or eventlet "
+                      "instead in version 'pike' and will be removed in "
+                      "version 'rocky'",
+                      category=FutureWarning, stacklevel=3)
+        ], warn.mock_calls)
 
     def test_server_wait_method(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(topic='foo', server='bar')
         endpoints = [object()]
         serializer = object()
@@ -157,7 +198,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self.assertEqual(1, listener.cleanup.call_count)
 
     def test_no_target_server(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         server = oslo_messaging.get_rpc_server(
             transport,
@@ -172,7 +213,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def test_no_server_topic(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
         target = oslo_messaging.Target(server='testserver')
         server = oslo_messaging.get_rpc_server(transport, target, [])
         try:
@@ -184,7 +225,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def _test_no_client_topic(self, call=True):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         client = self._setup_client(transport, topic=None)
 
@@ -205,7 +246,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._test_no_client_topic(call=False)
 
     def test_client_call_timeout(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         finished = False
         wait = threading.Condition()
@@ -233,7 +274,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_unknown_executor(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         try:
             oslo_messaging.get_rpc_server(transport, None, [], executor='foo')
@@ -244,7 +285,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertTrue(False)
 
     def test_cast(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def __init__(self):
@@ -265,14 +306,20 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self.assertEqual(['dsfoo', 'dsbar'], endpoint.pings)
 
     def test_call(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        # NOTE(milan): using a separate transport instance for each the client
+        # and the server to be able to check independent transport instances
+        # can communicate over same exchange&topic
+        transport_srv = oslo_messaging.get_rpc_transport(self.conf,
+                                                         url='fake:')
+        transport_cli = oslo_messaging.get_rpc_transport(self.conf,
+                                                         url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
                 return arg
 
-        server_thread = self._setup_server(transport, TestEndpoint())
-        client = self._setup_client(transport)
+        server_thread = self._setup_server(transport_srv, TestEndpoint())
+        client = self._setup_client(transport_cli)
 
         self.assertIsNone(client.call({}, 'ping', arg=None))
         self.assertEqual(0, client.call({}, 'ping', arg=0))
@@ -284,7 +331,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_direct_call(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
@@ -304,7 +351,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_context(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ctxt_check(self, ctxt, key):
@@ -321,7 +368,7 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         self._stop_server(client, server_thread)
 
     def test_failure(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         class TestEndpoint(object):
             def ping(self, ctxt, arg):
@@ -340,8 +387,10 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
                 a = a[0]
             errors.append(str(msg) % a)
 
-        self.stubs.Set(rpc_server_module.LOG, 'debug', stub_debug)
-        self.stubs.Set(rpc_server_module.LOG, 'error', stub_error)
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'debug', stub_debug))
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'error', stub_error))
 
         server_thread = self._setup_server(transport, TestEndpoint())
         client = self._setup_client(transport)
@@ -352,14 +401,14 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
             self.assertIsInstance(ex, ValueError)
             self.assertEqual('dsfoo', str(ex))
             self.assertTrue(len(debugs) == 0)
-            self.assertTrue(len(errors) > 0)
+            self.assertGreater(len(errors), 0)
         else:
             self.assertTrue(False)
 
         self._stop_server(client, server_thread)
 
     def test_expected_failure(self):
-        transport = oslo_messaging.get_transport(self.conf, url='fake:')
+        transport = oslo_messaging.get_rpc_transport(self.conf, url='fake:')
 
         debugs = []
         errors = []
@@ -374,8 +423,10 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
                 a = a[0]
             errors.append(str(msg) % a)
 
-        self.stubs.Set(rpc_server_module.LOG, 'debug', stub_debug)
-        self.stubs.Set(rpc_server_module.LOG, 'error', stub_error)
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'debug', stub_debug))
+        self.useFixture(fixtures.MockPatchObject(
+            rpc_server_module.LOG, 'error', stub_error))
 
         class TestEndpoint(object):
             @oslo_messaging.expected_exceptions(ValueError)
@@ -390,13 +441,27 @@ class TestRPCServer(test_utils.BaseTestCase, ServerSetupMixin):
         except Exception as ex:
             self.assertIsInstance(ex, ValueError)
             self.assertEqual('dsfoo', str(ex))
-            self.assertTrue(len(debugs) > 0)
+            self.assertGreater(len(debugs), 0)
             self.assertTrue(len(errors) == 0)
         else:
             self.assertTrue(False)
 
         self._stop_server(client, server_thread)
 
+    @mock.patch('oslo_messaging.rpc.server.LOG')
+    def test_warning_when_notifier_transport(self, log):
+        transport = oslo_messaging.get_notification_transport(self.conf)
+        target = oslo_messaging.Target(topic='foo', server='bar')
+        endpoints = [object()]
+        serializer = object()
+
+        oslo_messaging.get_rpc_server(transport, target,
+                                      endpoints, serializer=serializer)
+        log.warning.assert_called_once_with(
+            "Using notification transport for RPC. Please use "
+            "get_rpc_transport to obtain an RPC transport "
+            "instance.")
+
 
 class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
 
@@ -459,8 +524,8 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
             single_server = params['server1'] == params['server2']
             return not (single_topic and single_server)
 
-        # fanout to multiple servers on same topic and exchange
-        # each endpoint will receive both messages
+        # fanout to multiple servers on same topic and exchange each endpoint
+        # will receive both messages
         def fanout_to_servers(scenario):
             params = scenario[1]
             fanout = params['fanout1'] or params['fanout2']
@@ -497,14 +562,16 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
 
     def setUp(self):
         super(TestMultipleServers, self).setUp(conf=cfg.ConfigOpts())
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.FakeExchangeManager._exchanges',
+            new_value={}))
 
     def test_multiple_servers(self):
-        url1 = 'fake:///' + (self.exchange1 or '')
-        url2 = 'fake:///' + (self.exchange2 or '')
-
-        transport1 = oslo_messaging.get_transport(self.conf, url=url1)
-        if url1 != url2:
-            transport2 = oslo_messaging.get_transport(self.conf, url=url1)
+        transport1 = oslo_messaging.get_rpc_transport(self.conf,
+                                                      url='fake:')
+        if self.exchange1 != self.exchange2:
+            transport2 = oslo_messaging.get_rpc_transport(self.conf,
+                                                          url='fake:')
         else:
             transport2 = transport1
 
@@ -524,12 +591,18 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
             endpoint1 = endpoint2 = TestEndpoint()
 
         server1 = self._setup_server(transport1, endpoint1,
-                                     topic=self.topic1, server=self.server1)
+                                     topic=self.topic1,
+                                     exchange=self.exchange1,
+                                     server=self.server1)
         server2 = self._setup_server(transport2, endpoint2,
-                                     topic=self.topic2, server=self.server2)
+                                     topic=self.topic2,
+                                     exchange=self.exchange2,
+                                     server=self.server2)
 
-        client1 = self._setup_client(transport1, topic=self.topic1)
-        client2 = self._setup_client(transport2, topic=self.topic2)
+        client1 = self._setup_client(transport1, topic=self.topic1,
+                                     exchange=self.exchange1)
+        client2 = self._setup_client(transport2, topic=self.topic2,
+                                     exchange=self.exchange2)
 
         client1 = client1.prepare(server=self.server1)
         client2 = client2.prepare(server=self.server2)
@@ -545,9 +618,9 @@ class TestMultipleServers(test_utils.BaseTestCase, ServerSetupMixin):
         (client2.call if self.call2 else client2.cast)({}, 'ping', arg='2')
 
         self._stop_server(client1.prepare(fanout=None),
-                          server1, topic=self.topic1)
+                          server1, topic=self.topic1, exchange=self.exchange1)
         self._stop_server(client2.prepare(fanout=None),
-                          server2, topic=self.topic2)
+                          server2, topic=self.topic2, exchange=self.exchange2)
 
         def check(pings, expect):
             self.assertEqual(len(expect), len(pings))
@@ -861,3 +934,22 @@ class TestServerLocking(test_utils.BaseTestCase):
 
         # We timed out. Ensure we didn't log anything.
         self.assertFalse(mock_log.warning.called)
+
+
+class TestRPCExposeDecorator(test_utils.BaseTestCase):
+
+    def foo(self):
+        pass
+
+    @rpc.expose
+    def bar(self):
+        """bar docstring"""
+        pass
+
+    def test_undecorated(self):
+        self.assertRaises(AttributeError, lambda: self.foo.exposed)
+
+    def test_decorated(self):
+        self.assertEqual(True, self.bar.exposed)
+        self.assertEqual("""bar docstring""", self.bar.__doc__)
+        self.assertEqual('bar', self.bar.__name__)
