@@ -25,54 +25,51 @@ class TestParseURL(test_utils.BaseTestCase):
 
     scenarios = [
         ('transport',
-         dict(url='foo:', aliases=None,
-              expect=dict(transport='foo'))),
-        ('transport_aliased',
-         dict(url='bar:', aliases=dict(bar='foo'),
+         dict(url='foo:',
               expect=dict(transport='foo'))),
         ('virtual_host_slash',
-         dict(url='foo:////', aliases=None,
+         dict(url='foo:////',
               expect=dict(transport='foo', virtual_host='/'))),
         ('virtual_host',
-         dict(url='foo:///bar', aliases=None,
+         dict(url='foo:///bar',
               expect=dict(transport='foo', virtual_host='bar'))),
         ('host',
-         dict(url='foo://host/bar', aliases=None,
+         dict(url='foo://host/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(host='host'),
                           ]))),
         ('ipv6_host',
-         dict(url='foo://[ffff::1]/bar', aliases=None,
+         dict(url='foo://[ffff::1]/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(host='ffff::1'),
                           ]))),
         ('port',
-         dict(url='foo://host:1234/bar', aliases=None,
+         dict(url='foo://host:1234/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(host='host', port=1234),
                           ]))),
         ('ipv6_port',
-         dict(url='foo://[ffff::1]:1234/bar', aliases=None,
+         dict(url='foo://[ffff::1]:1234/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(host='ffff::1', port=1234),
                           ]))),
         ('username',
-         dict(url='foo://u@host:1234/bar', aliases=None,
+         dict(url='foo://u@host:1234/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(host='host', port=1234, username='u'),
                           ]))),
         ('password',
-         dict(url='foo://u:p@host:1234/bar', aliases=None,
+         dict(url='foo://u:p@host:1234/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
@@ -80,14 +77,14 @@ class TestParseURL(test_utils.BaseTestCase):
                                    username='u', password='p'),
                           ]))),
         ('creds_no_host',
-         dict(url='foo://u:p@/bar', aliases=None,
+         dict(url='foo://u:p@/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
                               dict(username='u', password='p'),
                           ]))),
         ('multi_host',
-         dict(url='foo://u:p@host1:1234,host2:4321/bar', aliases=None,
+         dict(url='foo://u:p@host1:1234,host2:4321/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
@@ -95,8 +92,16 @@ class TestParseURL(test_utils.BaseTestCase):
                                    username='u', password='p'),
                               dict(host='host2', port=4321),
                           ]))),
+        ('multi_host_partial_creds',
+         dict(url='foo://u:p@host1,host2/bar',
+              expect=dict(transport='foo',
+                          virtual_host='bar',
+                          hosts=[
+                              dict(host='host1', username='u', password='p'),
+                              dict(host='host2'),
+                          ]))),
         ('multi_creds',
-         dict(url='foo://u1:p1@host1:1234,u2:p2@host2:4321/bar', aliases=None,
+         dict(url='foo://u1:p1@host1:1234,u2:p2@host2:4321/bar',
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
@@ -107,7 +112,6 @@ class TestParseURL(test_utils.BaseTestCase):
                           ]))),
         ('multi_creds_ipv6',
          dict(url='foo://u1:p1@[ffff::1]:1234,u2:p2@[ffff::2]:4321/bar',
-              aliases=None,
               expect=dict(transport='foo',
                           virtual_host='bar',
                           hosts=[
@@ -117,7 +121,7 @@ class TestParseURL(test_utils.BaseTestCase):
                                    username='u2', password='p2'),
                           ]))),
         ('quoting',
-         dict(url='foo://u%24:p%26@host:1234/%24', aliases=None,
+         dict(url='foo://u%24:p%26@host:1234/%24',
               expect=dict(transport='foo',
                           virtual_host='$',
                           hosts=[
@@ -129,8 +133,7 @@ class TestParseURL(test_utils.BaseTestCase):
     def test_parse_url(self):
         self.config(rpc_backend=None)
 
-        url = oslo_messaging.TransportURL.parse(self.conf, self.url,
-                                                self.aliases)
+        url = oslo_messaging.TransportURL.parse(self.conf, self.url)
 
         hosts = []
         for host in self.expect.get('hosts', []):
@@ -154,35 +157,18 @@ class TestFormatURL(test_utils.BaseTestCase):
               transport=None,
               virtual_host=None,
               hosts=[],
-              aliases=None,
-              expected='testbackend:///')),
-        ('rpc_backend_aliased',
-         dict(rpc_backend='testfoo',
-              transport=None,
-              virtual_host=None,
-              hosts=[],
-              aliases=dict(testfoo='testbackend'),
               expected='testbackend:///')),
         ('transport',
          dict(rpc_backend=None,
               transport='testtransport',
               virtual_host=None,
               hosts=[],
-              aliases=None,
-              expected='testtransport:///')),
-        ('transport_aliased',
-         dict(rpc_backend=None,
-              transport='testfoo',
-              virtual_host=None,
-              hosts=[],
-              aliases=dict(testfoo='testtransport'),
               expected='testtransport:///')),
         ('virtual_host',
          dict(rpc_backend=None,
               transport='testtransport',
               virtual_host='/vhost',
               hosts=[],
-              aliases=None,
               expected='testtransport:////vhost')),
         ('host',
          dict(rpc_backend=None,
@@ -194,7 +180,6 @@ class TestFormatURL(test_utils.BaseTestCase):
                        username='bob',
                        password='secret'),
               ],
-              aliases=None,
               expected='testtransport://bob:secret@host:10//')),
         ('multi_host',
          dict(rpc_backend=None,
@@ -210,7 +195,6 @@ class TestFormatURL(test_utils.BaseTestCase):
                        username='b2',
                        password='s2'),
               ],
-              aliases=None,
               expected='testtransport://b1:s1@h1:1000,b2:s2@h2:2000/')),
         ('quoting',
          dict(rpc_backend=None,
@@ -222,7 +206,6 @@ class TestFormatURL(test_utils.BaseTestCase):
                        username='b$',
                        password='s&'),
               ],
-              aliases=None,
               expected='testtransport://b%24:s%26@host:10//%24')),
     ]
 
@@ -236,10 +219,7 @@ class TestFormatURL(test_utils.BaseTestCase):
                                                       host.get('username'),
                                                       host.get('password')))
 
-        url = oslo_messaging.TransportURL(self.conf,
-                                          self.transport,
-                                          self.virtual_host,
-                                          hosts,
-                                          self.aliases)
+        url = oslo_messaging.TransportURL(self.conf, self.transport,
+                                          self.virtual_host, hosts)
 
         self.assertEqual(self.expected, str(url))
