@@ -29,6 +29,7 @@ __all__ = [
 
 import logging
 
+from debtcollector import removals
 from oslo_config import cfg
 import six
 from six.moves.urllib import parse
@@ -42,8 +43,16 @@ LOG = logging.getLogger(__name__)
 _transport_opts = [
     cfg.StrOpt('transport_url',
                secret=True,
-               help='A URL representing the messaging driver to use and its '
-                    'full configuration.'),
+               help='The network address and optional user credentials for '
+                    'connecting to the messaging backend, in URL format. The '
+                    'expected format is:\n\n'
+                    'driver://[user:pass@]host:port[,[userN:passN@]hostN:'
+                    'portN]/virtual_host?query\n\n'
+                    'Example: rabbit://rabbitmq:password@127.0.0.1:5672//\n\n'
+                    'For full details on the fields in the URL see the '
+                    'documentation of oslo_messaging.TransportURL at '
+                    'https://docs.openstack.org/oslo.messaging/latest/'
+                    'reference/transport.html'),
     cfg.StrOpt('rpc_backend',
                deprecated_for_removal=True,
                deprecated_reason="Replaced by [DEFAULT]/transport_url",
@@ -76,8 +85,33 @@ class Transport(object):
     This is a mostly opaque handle for an underlying messaging transport
     driver.
 
-    It has a single 'conf' property which is the cfg.ConfigOpts instance used
-    to construct the transport object.
+    RPCs and Notifications may use separate messaging systems that utilize
+    different drivers, access permissions, message delivery, etc. To ensure
+    the correct messaging functionality, the corresponding method should be
+    used to construct a Transport object from transport configuration
+    gleaned from the user's configuration and, optionally, a transport URL.
+
+    The factory method for RPC Transport objects::
+
+        def get_rpc_transport(conf, url=None,
+                              allowed_remote_exmods=None)
+
+    If a transport URL is supplied as a parameter, any transport configuration
+    contained in it takes precedence. If no transport URL is supplied, but
+    there is a transport URL supplied in the user's configuration then that
+    URL will take the place of the URL parameter.
+
+    The factory method for Notification Transport objects::
+
+        def get_notification_transport(conf, url=None,
+                                       allowed_remote_exmods=None)
+
+    If no transport URL is provided, the URL in the notifications section of
+    the config file will be used.  If that URL is also absent, the same
+    transport as specified in the user's default section will be used.
+
+    The Transport has a single 'conf' property which is the cfg.ConfigOpts
+    instance used to construct the transport object.
     """
 
     def __init__(self, driver):
@@ -88,13 +122,15 @@ class Transport(object):
         self._driver.require_features(requeue=requeue)
 
     def _send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
-              retry=None):
+              call_monitor_timeout=None, retry=None):
         if not target.topic:
             raise exceptions.InvalidTarget('A topic is required to send',
                                            target)
         return self._driver.send(target, ctxt, message,
                                  wait_for_reply=wait_for_reply,
-                                 timeout=timeout, retry=retry)
+                                 timeout=timeout,
+                                 call_monitor_timeout=call_monitor_timeout,
+                                 retry=retry)
 
     def _send_notification(self, target, ctxt, message, version, retry=None):
         if not target.topic:
@@ -127,6 +163,20 @@ class Transport(object):
         self._driver.cleanup()
 
 
+class RPCTransport(Transport):
+    """Transport object for RPC."""
+
+    def __init__(self, driver):
+        super(RPCTransport, self).__init__(driver)
+
+
+class NotificationTransport(Transport):
+    """Transport object for notifications."""
+
+    def __init__(self, driver):
+        super(NotificationTransport, self).__init__(driver)
+
+
 class InvalidTransportURL(exceptions.MessagingException):
     """Raised if transport URL is invalid."""
 
@@ -145,7 +195,33 @@ class DriverLoadFailure(exceptions.MessagingException):
         self.ex = ex
 
 
-def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
+def _get_transport(conf, url=None, allowed_remote_exmods=None,
+                   transport_cls=RPCTransport):
+    allowed_remote_exmods = allowed_remote_exmods or []
+    conf.register_opts(_transport_opts)
+
+    if not isinstance(url, TransportURL):
+        url = TransportURL.parse(conf, url)
+
+    kwargs = dict(default_exchange=conf.control_exchange,
+                  allowed_remote_exmods=allowed_remote_exmods)
+
+    try:
+        mgr = driver.DriverManager('oslo.messaging.drivers',
+                                   url.transport.split('+')[0],
+                                   invoke_on_load=True,
+                                   invoke_args=[conf, url],
+                                   invoke_kwds=kwargs)
+    except RuntimeError as ex:
+        raise DriverLoadFailure(url.transport, ex)
+
+    return transport_cls(mgr.driver)
+
+
+@removals.remove(
+    message='use get_rpc_transport or get_notification_transport'
+)
+def get_transport(conf, url=None, allowed_remote_exmods=None):
     """A factory method for Transport objects.
 
     This method will construct a Transport object from transport configuration
@@ -166,34 +242,15 @@ def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
 
     :param conf: the user configuration
     :type conf: cfg.ConfigOpts
-    :param url: a transport URL
+    :param url: a transport URL, see :py:class:`transport.TransportURL`
     :type url: str or TransportURL
     :param allowed_remote_exmods: a list of modules which a client using this
                                   transport will deserialize remote exceptions
                                   from
     :type allowed_remote_exmods: list
-    :param aliases: A map of transport alias to transport name
-    :type aliases: dict
     """
-    allowed_remote_exmods = allowed_remote_exmods or []
-    conf.register_opts(_transport_opts)
-
-    if not isinstance(url, TransportURL):
-        url = TransportURL.parse(conf, url, aliases)
-
-    kwargs = dict(default_exchange=conf.control_exchange,
-                  allowed_remote_exmods=allowed_remote_exmods)
-
-    try:
-        mgr = driver.DriverManager('oslo.messaging.drivers',
-                                   url.transport.split('+')[0],
-                                   invoke_on_load=True,
-                                   invoke_args=[conf, url],
-                                   invoke_kwds=kwargs)
-    except RuntimeError as ex:
-        raise DriverLoadFailure(url.transport, ex)
-
-    return Transport(mgr.driver)
+    return _get_transport(conf, url, allowed_remote_exmods,
+                          transport_cls=RPCTransport)
 
 
 class TransportHost(object):
@@ -231,12 +288,40 @@ class TransportURL(object):
 
     Transport URLs take the form::
 
-      transport://user:pass@host:port[,userN:passN@hostN:portN]/virtual_host?query
+      driver://[user:pass@]host:port[,[userN:passN@]hostN:portN]/virtual_host?query
 
-    i.e. the scheme selects the transport driver, you may include multiple
-    hosts in netloc, the path part is a "virtual host" partition path and
-    the query part contains some driver-specific options which may override
-    corresponding values from a static configuration.
+    where:
+
+    driver
+      Specifies the transport driver to use. Typically this is `rabbit` for the
+      RabbitMQ broker. See the documentation for other available transport
+      drivers.
+
+    [user:pass@]host:port
+      Specifies the network location of the broker. `user` and `pass` are the
+      optional username and password used for authentication with the broker.
+
+      `user` and `pass` may contain any of the following ASCII characters:
+        * Alphabetic (a-z and A-Z)
+        * Numeric (0-9)
+        * Special characters: & = $ - _ . + ! * ( )
+
+      `user` may include at most one `@` character for compatibility with some
+      implementations of SASL.
+
+      All other characters in `user` and `pass` must be encoded via '%nn'
+
+      You may include multiple different network locations separated by commas.
+      The client will connect to any of the available locations and will
+      automatically fail over to another should the connection fail.
+
+    virtual_host
+      Specifies the "virtual host" within the broker. Support for virtual hosts
+      is specific to the message bus used.
+
+    query
+      Permits passing driver-specific options which override the corresponding
+      values from the configuration file.
 
     :param conf: a ConfigOpts instance
     :type conf: oslo.config.cfg.ConfigOpts
@@ -246,14 +331,12 @@ class TransportURL(object):
     :type virtual_host: str
     :param hosts: a list of TransportHost objects
     :type hosts: list
-    :param aliases: DEPRECATED: a map of transport alias to transport name
-    :type aliases: dict
     :param query: a dictionary of URL query parameters
     :type query: dict
     """
 
     def __init__(self, conf, transport=None, virtual_host=None, hosts=None,
-                 aliases=None, query=None):
+                 query=None):
         self.conf = conf
         self.conf.register_opts(_transport_opts)
         self._transport = transport
@@ -262,10 +345,6 @@ class TransportURL(object):
             self.hosts = []
         else:
             self.hosts = hosts
-        if aliases is None:
-            self.aliases = {}
-        else:
-            self.aliases = aliases
         if query is None:
             self.query = {}
         else:
@@ -279,10 +358,10 @@ class TransportURL(object):
             transport = self.conf.rpc_backend
         else:
             transport = self._transport
-        final_transport = self.aliases.get(transport, transport)
+        final_transport = transport
         if not self._deprecation_logged and final_transport != transport:
             # NOTE(sileht): The first step is deprecate this one cycle.
-            # To ensure deployer have updated they configuration during Octavia
+            # To ensure deployer have updated they configuration during Ocata
             # Then in P we will deprecate aliases kwargs of TransportURL() and
             # get_transport() for consuming application
             LOG.warning('legacy "rpc_backend" is deprecated, '
@@ -361,8 +440,9 @@ class TransportURL(object):
         return url
 
     @classmethod
-    def parse(cls, conf, url=None, aliases=None):
-        """Parse an url.
+    def parse(cls, conf, url=None):
+        """Parse a URL as defined by :py:class:`TransportURL` and return a
+        TransportURL object.
 
         Assuming a URL takes the form of::
 
@@ -390,14 +470,14 @@ class TransportURL(object):
         :type conf: oslo.config.cfg.ConfigOpts
         :param url: The URL to parse
         :type url: str
-        :param aliases: A map of transport alias to transport name
-        :type aliases: dict
         :returns: A TransportURL
         """
 
+        if not url:
+            conf.register_opts(_transport_opts)
         url = url or conf.transport_url
         if not url:
-            return cls(conf, aliases=aliases)
+            return cls(conf)
 
         if not isinstance(url, six.string_types):
             raise InvalidTransportURL(url, 'Wrong URL type')
@@ -411,7 +491,7 @@ class TransportURL(object):
 
         query = {}
         if url.query:
-            for key, values in six.iteritems(parse.parse_qs(url.query)):
+            for key, values in parse.parse_qs(url.query).items():
                 query[key] = ','.join(values)
 
         virtual_host = None
@@ -430,7 +510,7 @@ class TransportURL(object):
             username = password = port = None
 
             if '@' in host:
-                username, hostname = host.split('@', 1)
+                username, hostname = host.rsplit('@', 1)
                 if ':' in username:
                     username, password = username.split(':', 1)
                     password = parse.unquote(password)
@@ -453,9 +533,13 @@ class TransportURL(object):
                 # parses the port data
                 port = None
                 if ':' in port_text:
-                    port = int(port_text.split(':', 1)[1])
+                    port = port_text.split(':', 1)[1]
             elif ':' in hostname:
                 hostname, port = hostname.split(':', 1)
+
+            if port == "":
+                port = None
+            if port is not None:
                 port = int(port)
 
             if username is None or password is None:
@@ -477,4 +561,4 @@ class TransportURL(object):
                         {'hosts_with_credentials': hosts_with_credentials,
                          'hosts_without_credentials':
                          hosts_without_credentials})
-        return cls(conf, transport, virtual_host, hosts, aliases, query)
+        return cls(conf, transport, virtual_host, hosts, query)
